#####################################
# Example of analysis of data
#####################################

import numpy as np
import pylab as plt
from boutdata.collect import collect

path='./data/'
var=collect('P', path=path)

dcvar=np.mean(var, axis=3)
rmsvar=np.sqrt(np.mean(var**2,axis=3)-dcvar**2)

plt.figure()
plt.plot(rmsvar[:,34,32])
plt.show(block=False)
fvar=np.fft.rfft(var,axis=3)

plt.figure()
plt.plot(abs(fvar[:,34,32,1:10]))
plt.show(block=False)


plt.figure()
plt.semilogy(abs(fvar[:,34,32,1:7]))
plt.show(block=False)

plt.show()



# note - these commands are only run by default in interactive mode
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from builtins import filter
from builtins import range
from past.utils import old_div
from builtins import object
import os
import sys

try:
    boutpath = os.environ["BOUT_TOP"]
    pylibpath = boutpath + "tools/pylib"
    pbpath = pylibpath + "/post_bout"
    boutdatapath = pylibpath + "/boutdata"
    boututilpath = pylibpath + "/boututils"

    allpath = [boutpath, pylibpath, pbpath, boutdatapath, boututilpath]
    [sys.path.append(elem) for elem in allpath]

except:
    print("unable to append needed .py files")

sys.path.append("/usr/local/pylib")

import post_bout as post_bout
from .ListDict import ListDictKey, ListDictFilt
from .read_inp import parse_inp, read_inp, read_log
from .basic_info import weighted_avg_and_std
from .read_cxx import read_cxx, findlowpass


import os
import numpy as np
import pickle
import subprocess


def corral(
    cached=True, refresh=False, debug=False, IConly=1, logname="status.log", skew=False
):
    print("in corral")
    log = read_log(logname=logname)
    # done = log['done']
    runs = log["runs"]  # a list of all directories, we need this,
    # only need 'runs' if the simulation is done

    current = log["current"]  # always return the last data_dir

    print(log)
    print("current:", current)

    if refresh == True:
        for i, path in enumerate(runs):
            print(i, path)
            a = post_bout.save(path=path, IConly=IConly)  # re post-process a run

    elif (
        cached == False
    ):  # if all the ind. simulation pkl files are in place skip this part
        a = post_bout.save(path=current)  # save to current dir
        # here is really where you shoudl write to status.log
        # write_log('status.log',
        cached = True

    # if done:

    all_ave = []
    all_modes = []
    print("last_one: ")
    for i, val in enumerate(runs):
        print(val)
        mode_db, ave_db = post_bout.read(path=val)
        # alldata.append(array)
        all_modes.append(mode_db)
        all_ave.append(ave_db)

        # build the end database

        # remove the read in pickle

    # return alldb
    def islist(input):
        return isinstance(input, list)

    all_modes = list(filter(islist, all_modes))
    # all_ave = filter(islist,all_ave)

    # alldb = sum(alldb,[])
    # alldata = np.array(alldata)
    all_modes = sum(all_modes, [])

    nt = []
    for mode in all_modes:
        nt.append(len(mode["amp"]))
        nt = [max(nt)]

    nt = nt[0]
    t = list(range(nt))
    i = 0

    if debug:
        return all_modes, all_ave
    else:
        return LinRes(all_modes)


class LinRes(object):
    def __init__(self, all_modes):
        self.mode_db = all_modes
        self.db = all_modes
        # self.ave_db = all_ave

        alldb = self.db
        # self.modekeys = data[0]['fields']['Ni']['modes'][0].keys()
        # print len(alldb)

        self.meta = np.array(ListDictKey(self.db, "meta"))[0]

        self.keys = list((self.mode_db)[0].keys())
        # self.avekeys = data[0]['fields']['Ni']['ave'].keys()

        # self.nrun = len(alldb) #number of runs

        self.path = np.array(ListDictKey(self.db, "path"))
        self.cxx = []
        self.maxN = []

        self.ave = np.array(ListDictKey(alldb, "ave"))

        # [self.cxx.append(read_cxx(path=elem,boutcxx='2fluid.cxx.ref')) for elem in self.path]
        # [self.maxN.append(findlowpass(elem)) for elem in self.cxx]
        [
            self.cxx.append(read_cxx(path=elem, boutcxx="physics_code.cxx.ref"))
            for elem in self.path
        ]

        self.maxZ = np.array(ListDictKey(alldb, "maxZ"))
        self.maxN = self.maxZ
        # self.maxN = findlowpass(self.cxx) #low pass filt from .cxx

        self.nx = np.array(ListDictKey(alldb, "nx"))[0]
        self.ny = np.array(ListDictKey(alldb, "ny"))[0]
        self.nz = np.array(ListDictKey(alldb, "nz"))[0]

        # self.nt = int(data[0]['meta']['NOUT']['v']+1)
        self.Rxy = np.array(ListDictKey(alldb, "Rxy"))
        self.Rxynorm = np.array(ListDictKey(alldb, "Rxynorm"))
        self.nt = np.array(ListDictKey(alldb, "nt"))

        self.dt = np.array(ListDictKey(alldb, "dt"))
        self.nfields = np.array(ListDictKey(alldb, "nfields"))

        self.field = np.array(ListDictKey(alldb, "field"))

        self.k = np.array(ListDictKey(alldb, "k"))
        self.k_r = np.array(ListDictKey(alldb, "k_r"))

        self.mn = np.array(ListDictKey(alldb, "mn"))

        # return ListDictKey(alldb,'phase')

        # self.phase = np.array(ListDictKey(alldb,'phase'))
        self.phase = ListDictKey(alldb, "phase")

        # self.amp= np.array(ListDictKey(alldb,'amp'))
        # self.amp_n=np.array(ListDictKey(alldb,'amp_n'))
        # self.dc= []
        # self.freq = np.array(ListDictKey(alldb,'k'))
        # self.gamma = np.array(ListDictKey(alldb,'gamma'))

        self.amp = ListDictKey(alldb, "amp")
        self.amp_n = ListDictKey(alldb, "amp_n")
        self.dc = []
        # self.freq = np.array(ListDictKey(alldb,'k'))
        self.gamma = np.array(ListDictKey(alldb, "gamma"))
        self.gamma_i = np.array(ListDictKey(alldb, "gamma_i"))

        self.freq = np.array(ListDictKey(alldb, "freq"))

        self.IC = np.array(ListDictKey(alldb, "IC"))
        self.dz = np.array(ListDictKey(alldb, "dz"))
        self.meta["dz"] = np.array(list(set(self.dz).union()))

        self.nmodes = self.dz.size

        self.MN = np.array(ListDictKey(alldb, "MN"))
        # self.MN = np.float32(self.mn)
        # self.MN[:,1] = self.mn[:,1]/self.dz
        self.nrun = len(set(self.path).union())
        self.L = np.array(ListDictKey(alldb, "L"))
        # self.C_s =
        self.modeid = np.array(ListDictKey(alldb, "modeid"))

        self.trans = np.array(ListDictKey(alldb, "transform"))

        if np.any(self.trans):
            self.phase_r = ListDictKey(alldb, "phase_r")
            self.gamma_r = np.array(ListDictKey(alldb, "gamma_r"))
            self.amp_r = ListDictKey(alldb, "amp_r")
            self.freq_r = np.array(ListDictKey(alldb, "freq_r"))

        # try:
        #     self.model(haswak=False) #
        # except:
        #     self.M = 0

        # try:
        try:  # analytic model based on simple matrix
            self.models = []
            # self.models.append(_model(self)) #create a list to contain models
            self.models.append(
                _model(self, haswak=True, name="haswak")
            )  # another model
        # self.models.append(_model(self,haswak=True,name='haswak_0',m=0))
        # for Ln in range(10):
        #     Lval = 10**((.2*Ln -1)/10)
        #     #Lval = 10**(Ln-1)
        #     #Lval =
        #     print Lval
        #     self.models.append(_model(self,varL=True,name='varL'+str(Lval),Lval=Lval,haswak=True))

        except:
            self.M = 0

        try:  # analytic models based on user defined complex omega
            self.ref = []
            self.ref.append(_ref(self))
        # self.ref.append(_ref(self,haswak=False,name='drift'))
        # demand a complex omega to compare
        # self.ref.append(_ref(self,haswas=True,name='haswak'))
        except:
            self.ref = 0

    # self.models.append(_model(self,haswak2=True,name='haswak2'))

    # except:
    #     print 'FAIL'

    def _amp(self, tind, xind):
        # first select modes that actually have valid (tind,xind)
        # indecies
        # s = subset(self.db,'modeid',modelist)
        return np.array([self.amp[i][tind, xind] for i in range(self.nmodes)])

    # def model(self,field='Ni',plot=False,haswak=False):

    #    #enrich the object
    #    allk = self.k_r[:,1,self.nx/2] #one location for now
    #    allkpar = self.k_r[:,0,self.nx/2] #one location for now

    #    self.M = []
    #    self.eigsys = []
    #    self.gammaA = []
    #    self.omegaA = []
    #    self.eigvec = []
    #    self.gammamax = []
    #    self.omegamax = []

    #    #allk = np.arange(0.1,100.0,.1)
    #    #allk=  np.sort(list(set(allk).union()))

    #    for i,k in enumerate(allk):
    #       #print i
    #       #M =np.matrix(np.random.rand(3,3),dtype=complex)
    #       M = np.zeros([3,3],dtype=complex)
    #       M[0,0] = 0
    #       M[0,1] = k/(self.L[i,self.nx/2,self.ny/2])
    #       M[1,0] = (2*np.pi/self.meta['lpar'][self.nx/2])**2 * self.meta['sig_par'][0]*complex(0,k**-2)
    #       M[1,1]= -(2*np.pi/self.meta['lpar'][self.nx/2])**2 * self.meta['sig_par'][0]*complex(0,k**-2)

    #       if haswak:
    #           M[0,0] = M[0,0] + M[1,1]*complex(0,k**2)
    #           M[0,1] = M[0,1] + M[1,0]*complex(0,k**2)

    #       #if rho_conv:

    #       #M[1,0] = (allkpar[i])**2 * self.meta['sig_par'][0]*complex(0,k**-2)
    #       #M[1,1]= -(allkpar[i])**2 * self.meta['sig_par'][0]*complex(0,k**-2)

    #       eigsys= np.linalg.eig(M)
    #       gamma = (eigsys)[0].imag
    #       omega =(eigsys)[0].real
    #       eigvec = eigsys[1]

    #       self.M.append(M)
    #       self.eigsys.append(eigsys)
    #       self.gammaA.append(gamma)
    #       self.gammamax.append(max(gamma))
    #       where = ((gamma == gamma.max()) & (omega != 0))
    #       self.omegamax.append(omega[where[0]])
    #       self.eigvec.append(eigvec)
    #       self.omegaA.append(omega)

    class __model__(object):
        def __init__(self):
            self.M = 0


class subset(LinRes):
    def __init__(self, alldb, key, valuelist, model=False):
        selection = ListDictFilt(alldb, key, valuelist)
        if len(selection) != 0:
            LinRes.__init__(self, selection)
            self.skey = key
            if model == True:
                self.model()
        else:
            LinRes.__init__(self, alldb)
            if model == True:
                self.model()


# class subset(originClass):
#     def __init__(self,alldb,key,valuelist,model=False):
#       selection = ListDictFilt(alldb,key,valuelist)
#       if len(selection) !=0:
#          originClass.__init__(self,selection,input_obj.ave_db)
#          self.skey = key
#          if model==True:
#             self.model()
#       else:
#          origin.__init__(self,alldb)
#          if model==True:
#             self.model()

# not sure if this is the best way . . .


# class subset(object):
#     def __init__(self,input_obj,key,valuelist,model=False):
#         selection = ListDictFilt(input_obj.mode_db,key,valuelist)
#         if len(selection) !=0:
#             import copy
#             self = copy.copy(input_obj)
#             self.__init__(selection,input_obj.ave_db)
#             self.skey = key
#             if model==True:
#                 self.model()
#         else:
#             self = input_obj
#             if model==True:
#                 self.model()


class _ref(object):  # NOT a derived obj, just takes one as a var
    def __init__(self, input_obj, name="haswak", haswak=True):
        allk = input_obj.k_r[:, 1, old_div(input_obj.nx, 2)]  # one location for now
        allkpar = input_obj.k_r[:, 0, old_div(input_obj.nx, 2)]  # one location for now
        self.name = name

        self.gamma = []
        self.omega = []
        self.soln = {}
        self.soln["gamma"] = []
        self.soln["freq"] = []

        for i, k in enumerate(allk):
            omega_star = old_div(
                -(k),
                (input_obj.L[i, old_div(input_obj.nx, 2), old_div(input_obj.ny, 2)]),
            )

            nu = (
                2 * np.pi / input_obj.meta["lpar"][old_div(input_obj.nx, 2)]
            ) ** 2 * input_obj.meta["sig_par"][0]

            if haswak:
                omega = old_div(-omega_star, (1 + (k) ** 2))
                gamma = old_div(((k**2) * omega_star**2), (nu * (1 + k**2) ** 3))
            else:
                # omega = -np.sqrt(nu*omega_star)/(np.sqrt(2)*k) + nu**(3/2)/(8*np.sqrt(2*omega_star)*k**3)
                # gamma = np.sqrt(nu*omega_star)/(np.sqrt(2)*k) - nu/(2* k**2) + nu**(3/2)/(8*np.sqrt(2*omega_star)*k**3)
                omega = -omega_star + old_div((2 * k**4 * omega_star**3), nu**2)
                gamma = old_div((k * omega_star) ** 2, nu) - (
                    5 * (k**6 * omega_star * 4 / nu**3)
                )
            self.gamma.append(gamma)
            self.omega.append(omega)

        self.soln["freq"] = np.transpose(np.array(self.omega))
        self.soln["gamma"] = np.transpose(np.array(self.gamma))


class _model(object):  # NOT a derived class,but one that takes a class as input
    def __init__(
        self,
        input_obj,
        name="drift",
        haswak=False,
        rho_conv=False,
        haswak2=False,
        varL=False,
        Lval=1.0,
        m=1,
    ):
        allk = input_obj.k_r[:, 1, old_div(input_obj.nx, 2)]  # one location for now
        allkpar = input_obj.k_r[:, 0, old_div(input_obj.nx, 2)]  # one location for now

        # numerical value to compare against

        numgam = input_obj.gamma[:, 0, old_div(input_obj.nx, 2)]
        numfreq = input_obj.freq[:, 0, old_div(input_obj.nx, 2)]
        self.name = name

        self.M = []
        self.eigsys = []
        self.gammaA = []
        self.omegaA = []
        self.eigvec = []
        self.gammamax = []
        self.omegamax = []
        self.k = []
        self.m = m

        self.soln = {}
        self.soln["freq"] = []
        self.soln["gamma"] = []
        self.soln["gammamax"] = []
        self.soln["freqmax"] = []

        self.chi = {}
        self.chi["freq"] = []
        self.chi["gamma"] = []

        for i, k in enumerate(allk):
            # print i
            # M =np.matrix(np.random.rand(3,3),dtype=complex)
            M = np.zeros([4, 4], dtype=complex)
            M[0, 0] = 0
            # k = k/np.sqrt(10)
            # L = (input_obj.L)*np.sqrt(10)

            if k == 0:
                k = 1e-5

            # print k {n,phi,v,ajpar}
            M[0, 1] = old_div(
                k, (input_obj.L[i, old_div(input_obj.nx, 2), old_div(input_obj.ny, 2)])
            )
            M[1, 0] = (
                (2 * m * np.pi / input_obj.meta["lpar"][old_div(input_obj.nx, 2)]) ** 2
                * input_obj.meta["sig_par"][0]
                * complex(0, (k) ** -2)
            )
            M[1, 1] = (
                -(
                    (2 * m * np.pi / input_obj.meta["lpar"][old_div(input_obj.nx, 2)])
                    ** 2
                )
                * input_obj.meta["sig_par"][0]
                * complex(0, (k) ** -2)
            )

            # parallel dynamics
            # M[2,2] = k/(input_obj.L[i,input_obj.nx/2,input_obj.ny/2])
            # M[2,0] = -(2*m*np.pi/input_obj.meta['lpar'][input_obj.nx/2])
            # M[0,2] = -(2*m*np.pi/input_obj.meta['lpar'][input_obj.nx/2])

            # M[1,0] = (2*m*np.pi/input_obj.meta['lpar'][input_obj.nx/2])**2 * input_obj.meta['sig_par'][0]
            # M[1,1]= -(2*m*np.pi/input_obj.meta['lpar'][input_obj.nx/2])**2 * input_obj.meta['sig_par'][0]

            # ajpar dynamics - effectively parallel electron dynamics instead of

            if haswak:
                M[0, 0] = (
                    -(
                        (
                            2
                            * m
                            * np.pi
                            / input_obj.meta["lpar"][old_div(input_obj.nx, 2)]
                        )
                        ** 2
                    )
                    * input_obj.meta["sig_par"][0]
                    * complex(0, 1)
                )
                M[0, 1] = (
                    2 * m * np.pi / input_obj.meta["lpar"][old_div(input_obj.nx, 2)]
                ) ** 2 * input_obj.meta["sig_par"][0] * complex(0, 1) + M[0, 1]

            if varL:
                M[0, 1] = Lval * M[0, 1]

            if rho_conv:  # not used
                M[1, 0] = (
                    (allkpar[i]) ** 2
                    * input_obj.meta["sig_par"][0]
                    * complex(0, (k) ** -2)
                )
                M[1, 1] = (
                    -((allkpar[i]) ** 2)
                    * input_obj.meta["sig_par"][0]
                    * complex(0, (k) ** -2)
                )

            eigsys = np.linalg.eig(M)
            gamma = (eigsys)[0].imag
            omega = (eigsys)[0].real
            eigvec = eigsys[1]
            self.k.append(k)

            self.M.append(M)
            self.eigsys.append(eigsys)

            self.gammaA.append(gamma)
            self.soln["gamma"].append(gamma)

            self.gammamax.append(max(gamma))
            self.soln["gammamax"].append(max(gamma))

            where = (gamma == gamma.max()) & (omega != 0)
            # if len(where) > 1:
            #     where = where[0]
            self.omegamax.append(omega[where])
            self.soln["freqmax"].append(omega[where])

            # print k,gamma,where,M,omega
            chigam = old_div(((numgam - max(gamma)) ** 2), max(gamma))
            chifreq = old_div(((numfreq - omega[where]) ** 2), omega[where])

            self.eigvec.append(eigvec)
            self.omegaA.append(omega)
            self.soln["freq"].append(omega)

            self.chi["freq"].append(chifreq[i])

            self.chi["gamma"].append(chigam[i])

        self.dim = M.shape[0]
        self.soln["freq"] = np.transpose(np.array(self.soln["freq"]))
        self.soln["gamma"] = np.transpose(np.array(self.soln["gamma"]))
        self.chi["freq"] = np.transpose(np.array(self.chi["freq"]))
        self.chi["gamma"] = np.transpose(np.array(self.chi["gamma"]))

        # self.soln = {}
        # self.soln['freq'] = self.omegaA
        # self.soln['gamma'] = self.gammaA



from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div

# some standard analytic stuff to plot, if appending just overplot gam or omeg
from .pb_corral import LinRes
from .ListDict import ListDictKey, ListDictFilt
import numpy as np

import matplotlib.pyplot as plt
from matplotlib import cm
import matplotlib.artist as artist
import matplotlib.ticker as ticker
import matplotlib.pyplot as plt
import matplotlib.patches as patches
from matplotlib.figure import Figure
from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
from matplotlib.backends.backend_pdf import PdfPages

from reportlab.platypus import *
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.rl_config import defaultPageSize
from reportlab.lib.units import inch
from reportlab.graphics.charts.linecharts import HorizontalLineChart
from reportlab.graphics.shapes import Drawing
from reportlab.graphics.charts.lineplots import LinePlot
from reportlab.graphics.widgets.markers import makeMarker
from reportlab.lib import colors

from replab_x_vs_y import RL_Plot
from matplotlib.ticker import ScalarFormatter, FormatStrFormatter, MultipleLocator


class LinResDraw(LinRes):
    def __init__(self, alldb):
        LinRes.__init__(self, alldb)

    def plottheory(
        self, pp, m=1, canvas=None, comp="gamma", field="Ni", allroots=False
    ):
        if len(self.models) == 0:
            try:
                self.models = []
                self.models.append(_model(self))  # create a list to contain models
                self.models.append(
                    _model(self, haswak=True, name="haswak")
                )  # another model
            except:
                return 0

        s = subset(self.db, "field", [field])

        modelist = []

        [modelist.append([m, n + 1]) for n in range(min(s.maxN) - 1)]

        s = subset(s.db, "mn", modelist)

        allk = s.k[:, 1, old_div(s.nx, 2)]
        ki = np.argsort(allk)

        ownpage = False
        if canvas is None:
            ownpage = True

        if ownpage:  # if not an overplot
            fig1 = plt.figure()
            canvas = fig1.add_subplot(1, 1, 1)

        label = "gamma analytic"

        # if comp=='gamma':
        #     y = np.array(s.gammamax)[ki]
        # else:
        #     y = np.array(s.omegamax)[ki]

        for m in s.models:
            print(m.name)

        for i, m in enumerate(s.models):
            print(m.name, comp, m.soln[comp].shape)

            if allroots:
                for elem in m.soln[comp]:
                    # y = []
                    # elem has 2 or more elements
                    y = (np.array(elem)[ki]).flatten()  # n values
                    # y = y.astype('float')
                    print(y.shape)
                    canvas.plot(
                        (allk[ki]).flatten(), y, ",", label=label, c=cm.jet(0.2 * i)
                    )
            try:
                ymax = (np.array(m.soln[comp + "max"])[ki]).flatten()
                # ymax =  (np.array(m.gammamax)[ki]).flatten()
                ymax = ymax.astype("float")
                print(comp, " ymax:", ymax)
                canvas.plot(
                    (allk[ki]).flatten(), ymax, "-", label=label, c=cm.jet(0.2 * i)
                )
                # if comp=='gamma':
                #     y = (np.array(m.gammamax)[ki]).flatten()

                # else:
                #     y = (np.array(m.omegamax)[ki]).flatten()

            # print m.name, ':' ,y.astype('float')

            except:
                print("fail to add theory curve")

            canvas.annotate(m.name, (allk[ki[0]], 1.1 * ymax[0]), fontsize=8)
            canvas.annotate(m.name, (1.1 * allk[ki[-1]], 1.1 * ymax[-1]), fontsize=8)

            try:
                for i, m in enumerate(s.ref):
                    if not allroots:
                        y = (np.array(m.soln[comp])[ki]).flatten()
                        y = y.astype("float")
                        canvas.plot(
                            (allk[ki]).flatten(),
                            y,
                            "--",
                            label=label,
                            c=cm.jet(0.2 * i),
                        )
            except:
                print("no reference curve")

        if ownpage:  # set scales if this is its own plot
            # canvas.set_yscale('symlog',linthreshy=1e-13)
            # canvas.set_xscale('log')

            canvas.axis("tight")
            canvas.set_xscale("log")
            canvas.set_yscale("symlog")
            fig1.savefig(pp, format="pdf")
            plt.close(fig1)
        else:  # if not plot its probably plotted iwth sim data, print chi somewhere
            for i, m in enumerate(s.models):
                textstr = r"$\chi^2$" + "$=%.2f$" % (m.chi[comp].sum())
                print(textstr)
                # textstr = '$\L=%.2f$'%(m.chi[comp].sum())
                props = dict(boxstyle="square", facecolor="white", alpha=0.3)
                textbox = canvas.text(
                    0.1,
                    0.1,
                    textstr,
                    transform=canvas.transAxes,
                    fontsize=10,
                    verticalalignment="top",
                    bbox=props,
                )

    def plotomega(
        self,
        pp,
        canvas=None,
        field="Ni",
        yscale="linear",
        clip=0,
        xaxis="t",
        xscale="linear",
        xrange=1,
        comp="gamma",
        pltlegend="both",
        overplot=False,
        gridON=True,
        trans=False,
        infobox=True,
    ):
        colors = [
            "b.",
            "r.",
            "k.",
            "c.",
            "g.",
            "y.",
            "m.",
            "b.",
            "r.",
            "k.",
            "c.",
            "g.",
            "y.",
            "m.",
        ]
        colordash = [
            "b",
            "r",
            "k",
            "c",
            "g",
            "y",
            "m",
            "b",
            "r",
            "k",
            "c",
            "g",
            "y",
            "m",
        ]

        if canvas is None:
            ownpage = True
        else:
            ownpage = False

        if ownpage:
            fig1 = plt.figure()
            fig1.subplots_adjust(bottom=0.12)
            fig1.subplots_adjust(top=0.80)
            fig1.subplots_adjust(right=0.83)
            fig1.subplots_adjust(left=0.17)
            canvas = fig1.add_subplot(1, 1, 1)
            clonex = canvas.twinx()
            # if trans:
            #     cloney = canvas.twiny()

        dzhandles = []
        parhandles = []
        parlabels = []
        dzlabels = []

        m_shift = 1
        for q in np.array(list(range(1))) + m_shift:
            s = subset(self.db, "field", [field])  # pick  field
            maxZ = min(s.maxN)
            modelist = []
            [modelist.append([q, p + 1]) for p in range(maxZ - 1)]
            print(modelist)
            print(q, "in plotgamma")

        s = subset(s.db, "mn", modelist)

        xrange = old_div(s.nx, 2) - 2

        xrange = [old_div(s.nx, 2), old_div(s.nx, 2) + xrange]

        y = np.array(ListDictKey(s.db, comp))

        # y = s.gamma #nmodes x 2 x nx ndarray
        k = s.k  ##nmodes x 2 x nx ndarray k_zeta

        kfactor = np.mean(
            old_div(s.k_r[:, 1, old_div(s.nx, 2)], s.k[:, 1, old_div(s.nx, 2)])
        )  # good enough for now

        print(
            k[:, 1, old_div(s.nx, 2)].shape,
            y[:, 0, old_div(s.nx, 2)].shape,
            len(colors),
            ownpage,
        )  # ,k[:,1,s.nx/2],y[:,0,s.nx/2]

        parhandles.append(
            canvas.errorbar(
                np.squeeze(k[:, 1, old_div(s.nx, 2)]),
                np.squeeze(y[:, 0, old_div(s.nx, 2)]),
                yerr=np.squeeze(y[:, 1, old_div(s.nx, 2)]),
                fmt=colors[q],
            )
        )

        parlabels.append("m " + str(q))

        # loop over dz sets and connect with dotted line  . . .
        jj = 0

        ymin_data = np.max(np.array(ListDictKey(s.db, comp)))
        ymax_data = 0  # for bookeeping

        for p in list(set(s.path).union()):
            print(p, "in plotomega")

            sub_s = subset(s.db, "path", [p])
            j = sub_s.dz[0]
            # print sub_s.amp.shape
            s_i = np.argsort(sub_s.mn[:, 1])  # sort by 'local' m, global m is ok also
            # print s_i, sub_s.mn, sub_s.nx, jj
            y = np.array(ListDictKey(sub_s.db, comp))
            y_alt = 2.0 * np.array(ListDictKey(sub_s.db, comp))

            k = sub_s.k  ##
            if q == m_shift:  # fix the parallel mode
                dzhandles.append(
                    canvas.plot(
                        k[s_i, 1, old_div(sub_s.nx, 2)],
                        y[s_i, 0, old_div(sub_s.nx, 2)],
                        color=colordash[jj],
                        alpha=0.5,
                    )
                )
                # clonex.plot(k[s_i,1,sub_s.nx/2],
                # y_alt[s_i,0,sub_s.nx/2],color=colordash[jj],alpha=.5)
                # if np.any(sub_s.trans) and trans:
                #     comp_r = comp+'_r'
                #     k_r =  sub_s.k_r
                #     y2 = np.array(ListDictKey(sub_s.db,comp_r))
                #     cloney.plot(k[s_i,1,sub_s.nx/2],
                #                 y2[s_i,0,sub_s.nx/2],'k.',ms = 3)

                ymin_data = np.min([np.min(y[s_i, 0, old_div(sub_s.nx, 2)]), ymin_data])
                ymax_data = np.max([np.max(y[s_i, 0, old_div(sub_s.nx, 2)]), ymax_data])

                print("dzhandle color", jj)
                # dzlabels.append("DZ: "+ str(2*j)+r'$\pi$')
                dzlabels.append(j)

                if yscale == "log":
                    factor = 10
                else:
                    factor = 2
                print("annotating")
                canvas.annotate(
                    str(j),
                    (
                        k[s_i[0], 1, old_div(sub_s.nx, 2)],
                        y[s_i[0], 0, old_div(sub_s.nx, 2)],
                    ),
                    fontsize=8,
                )
                p = canvas.axvspan(
                    k[s_i[0], 1, old_div(sub_s.nx, 2)],
                    k[s_i[-1], 1, old_div(sub_s.nx, 2)],
                    facecolor=colordash[jj],
                    alpha=0.01,
                )
                print("done annotating")
            else:
                canvas.plot(
                    k[s_i, 1, old_div(sub_s.nx, 2)],
                    y[s_i, 0, old_div(sub_s.nx, 2)],
                    color=colordash[jj],
                    alpha=0.3,
                )

            jj = jj + 1

        dzhandles = np.array(dzhandles).flatten()
        dzlabels = np.array(dzlabels).flatten()

        dzlabels = list(set(dzlabels).union())

        dz_i = np.argsort(dzlabels)

        dzhandles = dzhandles[dz_i]
        dzlabels_cp = np.array(dzlabels)[dz_i]

        print(type(dzlabels), np.size(dzlabels))
        for i in range(np.size(dzlabels)):
            dzlabels[i] = "DZ: " + str(dzlabels_cp[i])  # +r"$\pi$"

        parlabels = np.array(parlabels).flatten()

        # if pltlegend =='both':    #

        print("legends")

        # l1 = legend(parhandles,parlabels,loc = 3,prop={'size':6})
        # l2 = legend(dzhandles,dzlabels,loc = 1,prop={'size':6})
        # plt.gca().add_artist(l1)

        # else:
        #    legend(dzhandles,dzlabels,loc=3,prop={'size':6})
        if overplot == True:
            try:
                self.plottheory(pp, canvas=canvas, comp=comp, field=field)
                # self.plottheory(pp,comp=comp)
            except:
                print("no theory plot")
        if infobox:
            textstr = "$\L_{\parallel}=%.2f$\n$\L_{\partial_r n}=%.2f$\n$B=%.2f$" % (
                s.meta["lpar"][old_div(s.nx, 2)],
                s.meta["L"][old_div(s.nx, 2), old_div(s.ny, 2)],
                s.meta["Bpxy"]["v"][old_div(s.nx, 2), old_div(s.ny, 2)],
            )
            props = dict(boxstyle="square", facecolor="white", alpha=0.3)
            textbox = canvas.text(
                0.82,
                0.95,
                textstr,
                transform=canvas.transAxes,
                fontsize=10,
                verticalalignment="top",
                bbox=props,
            )
            # leg = canvas.legend(handles,labels,ncol=2,loc='best',prop={'size':4},fancybox=True)
            # textbox.get_frame().set_alpha(0.3)
            # matplotlib.patches.Rectangle
            # p = patches.Rectangle((0, 0), 1, 1, fc="r")
            # p = str('L_par')
            # leg = canvas.legend([p], ["Red Rectangle"],loc='best',prop={'size':4})
            # leg.get_frame().set_alpha(0.3)

        # cloney.set_xlim(xmin,xmax)
        try:
            canvas.set_yscale(yscale)
            canvas.set_xscale(xscale)

            if yscale == "symlog":
                canvas.set_yscale(yscale, linthreshy=1e-13)
            if xscale == "symlog":
                canvas.set_xscale(xscale, linthreshy=1e-13)

            if gridON:
                canvas.grid()
        except:
            try:
                canvas.set_yscale("symlog")
            except:
                print("scaling failed completely")

        # print '[xmin, xmax, ymin, ymax]: ',[xmin, xmax, ymin, ymax]

        clonex.set_yscale(yscale)  # must be called before limits are set

        try:
            if yscale == "linear":
                formatter = ticker.ScalarFormatter()
                formatter.set_powerlimits((-2, 2))  # force scientific notation
                canvas.yaxis.set_major_formatter(formatter)
                clonex.yaxis.set_major_formatter(formatter)
                # canvas.useOffset=False
        except:
            print("fail 1")
        [xmin, xmax, ymin, ymax] = canvas.axis()

        if yscale == "symlog":
            clonex.set_yscale(yscale, linthreshy=1e-9)
        if xscale == "symlog":
            clonex.set_xscale(xscale, linthreshy=1e-9)
            # if np.any(s.trans) and trans:
        [xmin1, xmax1, ymin1, ymax1] = canvas.axis()
        if trans:
            try:
                cloney = canvas.twiny()
                # cloney.set_yscale(yscale)
                cloney.set_xscale(xscale)
                [xmin1, xmax1, ymin2, ymax2] = canvas.axis()

                if xscale == "symlog":
                    cloney.set_xscale(xscale, linthreshy=1e-9)
                if yscale == "symlog":
                    cloney.set_yscale(yscale, linthreshy=1e-9)
                if yscale == "linear":
                    cloney.yaxis.set_major_formatter(formatter)
            except:
                print("fail trans")
                #     cloney.useOffset=False

            # if xscale =='symlog' and trans:
            #     cloney.set_yscale(yscale,linthreshy=1e-9)
            #     cloney.set_xscale(xscale,linthreshy=1e-9)

        Ln_drive_scale = s.meta["w_Ln"][0] ** -1
        # Ln_drive_scale = 2.1e3
        clonex.set_ylim(Ln_drive_scale * ymin, Ln_drive_scale * ymax)

        try:
            if trans:
                # k_factor = #scales from k_zeta to k_perp
                cloney.set_xlim(kfactor * xmin, kfactor * xmax)
                # if np.any(sub_s.trans) and trans:
                #     comp_r = comp+'_r'
                #     y2 = np.array(ListDictKey(sub_s.db,comp_r))
                #    #canvas.plot(k[s_i,1,sub_s.nx/2],
                #    #           y2[s_i,0,sub_s.nx/2],'k.',ms = 3)

                #     cloney.plot(k_r[s_i,1,sub_s.nx/2],
                #                y2[s_i,0,sub_s.nx/2],'k.',ms = 3)
                #     print 'np.sum(np.abs(y-y2)): ',np.sum(np.abs(y-y2)),comp_r
                # kfactor =1.0
                # cloney.set_xlim(xmin,xmax)
                cloney.set_ylim(
                    ymin, ymax
                )  # because cloney shares the yaxis with canvas it may overide them, this fixes that
                cloney.set_xlabel(r"$k_{\perp} \rho_{ci}$", fontsize=18)
        except:
            print("moar fail")
            # clonex.set_xscale(xscale)

        # except:
        #     #canvas.set_xscale('symlog', linthreshx=0.1)
        #     print 'extra axis FAIL'

        # if yscale == 'linear':
        # canvas.yaxis.set_major_locator(ticker.LinearLocator(numticks=8))

        # minorLocator   = MultipleLocator(.005)
        # canvas.yaxis.set_minor_locator(minorLocator)
        # spawn another y label

        # clone = canvas.twinx()
        # s2 = np.sin(2*np.pi*t)
        # ax2.plot(x, s2, 'r.')

        # ion_acoust_str = r"$\frac{c_s}{L_{\partial_r n}}}$"

        if comp == "gamma":
            canvas.set_ylabel(
                r"$\frac{\gamma}{\omega_{ci}}$", fontsize=18, rotation="horizontal"
            )
            clonex.set_ylabel(
                r"$\frac{\gamma}{\frac{c_s}{L_n}}$",
                color="k",
                fontsize=18,
                rotation="horizontal",
            )
        if comp == "freq":
            canvas.set_ylabel(
                r"$\frac{\omega}{\omega_{ci}}$", fontsize=18, rotation="horizontal"
            )
            clonex.set_ylabel(
                r"$\frac{\omega}{\frac{c_s}{L_n}}$",
                color="k",
                fontsize=18,
                rotation="horizontal",
            )

        if comp == "amp":
            canvas.set_ylabel(r"$A_k$", fontsize=18, rotation="horizontal")
            clonex.set_ylabel(
                r"$\frac{A_k}{A_{max}}$", color="k", fontsize=18, rotation="horizontal"
            )

        canvas.set_xlabel(r"$k_{\zeta} \rho_{ci}$", fontsize=18)

        title = comp + " computed from " + field
        # canvas.set_title(title,fontsize=14)
        fig1.suptitle(title, fontsize=14)

        if ownpage:
            try:
                fig1.savefig(pp, format="pdf")
            except:
                print("pyplt doesnt like you")
            plt.close(fig1)

    def plotfreq(
        self, pp, field="Ni", clip=0, xaxis="t", xscale="linear", yscale="linear"
    ):
        # colors = ['b','g','r','c','m','y','k','b','g','r','c','m','y','k']
        colors = [
            "b.",
            "g.",
            "r.",
            "c.",
            "m.",
            "y.",
            "k.",
            "b.",
            "g.",
            "r.",
            "c.",
            "m.",
            "y",
            "k",
        ]
        plt.figure()

        # s = subset(self.db,'field',[field]) #pick  field

        for q in range(4):
            s = subset(self.db, "field", [field])  # pick  field across all dz sets
            modelist = []
            [modelist.append([q + 1, p + 1]) for p in range(5)]
            print(q, "in plotgamma")
            s = subset(s.db, "mn", modelist)

            gamma = s.freq  # nmodes x 2 x nx ndarray
            k = s.k  ##nmodes x 2 x nx ndarray

            plt.errorbar(
                k[:, 1, old_div(s.nx, 2)],
                gamma[:, 0, old_div(s.nx, 2)],
                yerr=gamma[:, 1, old_div(s.nx, 2)],
                fmt=colors[q],
            )
            plt.plot(
                k[:, 1, old_div(s.nx, 2)],
                gamma[:, 0, old_div(s.nx, 2)],
                "k:",
                alpha=0.3,
            )

            # loop over dz sets and connect with dotted line  . . .
            for j in list(set(s.dz).union()):
                # print j,len(s.mn)
                sub_s = subset(s.db, "dz", [j])
                gamma = sub_s.gamma
                k = sub_s.k  ##
                plt.plot(
                    k[:, 1, old_div(sub_s.nx, 2)],
                    gamma[:, 0, old_div(sub_s.nx, 2)],
                    "k:",
                    alpha=0.1,
                )

        try:
            plt.yscale(yscale)
        except:
            print("yscale fail")

        try:
            plt.xscale(yscale)
        except:
            plt.xscale("symlog")
        plt.xlabel(r"$k \rho_{ci}$", fontsize=14)
        plt.ylabel(r"$\frac{\omega}{\omega_{ci}}$", fontsize=14)
        # plt.title(r'$\frac{\omega}\{\omega_{ci}}$ '+ 'computed from'+field+ 'field',fontsize=10)

        plt.savefig(pp, format="pdf")
        plt.close()

    def plotgamma(
        self,
        pp,
        field="Ni",
        yscale="symlog",
        clip=0,
        xaxis="t",
        xscale="linear",
        xrange=1,
        comp="gamma",
        overplot=False,
        trans=True,
    ):
        self.plotomega(
            pp,
            field=field,
            yscale=yscale,
            clip=clip,
            xaxis=xaxis,
            xscale=xscale,
            xrange=xrange,
            comp=comp,
            overplot=overplot,
            trans=trans,
        )

    def plotfreq2(
        self,
        pp,
        field="Ni",
        yscale="symlog",
        clip=0,
        xaxis="t",
        xscale="linear",
        xrange=1,
        comp="freq",
        overplot=False,
        trans=True,
    ):
        self.plotomega(
            pp,
            field=field,
            yscale=yscale,
            clip=clip,
            xaxis=xaxis,
            xscale=xscale,
            xrange=xrange,
            comp=comp,
            overplot=overplot,
            trans=trans,
        )

    def plotvsK(
        self,
        pp,
        rootfig=None,
        field="Ni",
        yscale="log",
        clip=0,
        xaxis="t",
        xscale="linear",
        xrange=1,
        comp="amp",
        pltlegend="both",
        overplot=False,
        gridON=True,
        trans=False,
        infobox=True,
        m=1,
        t=[0],
        file=None,
        save=True,
    ):
        colors = [
            "b.",
            "r.",
            "k.",
            "c.",
            "g.",
            "y.",
            "m.",
            "b.",
            "r.",
            "k.",
            "c.",
            "g.",
            "y.",
            "m.",
        ]
        colordash = [
            "b",
            "r",
            "k",
            "c",
            "g",
            "y",
            "m",
            "b",
            "r",
            "k",
            "c",
            "g",
            "y",
            "m",
        ]

        if rootfig is None:
            ownpage = True
        else:
            ownpage = False

        if ownpage:
            fig1 = plt.figure()
            fig1.subplots_adjust(bottom=0.12)
            fig1.subplots_adjust(top=0.80)
            fig1.subplots_adjust(right=0.83)
            fig1.subplots_adjust(left=0.17)
            canvas = fig1.add_subplot(1, 1, 1)
            clonex = canvas.twinx()
            # if trans:
            #     cloney = canvas.twiny()
        else:
            canvas = rootfig.add_subplot(1, 1, 1)

        dzhandles = []
        parhandles = []
        parlabels = []
        dzlabels = []

        # pick the modes
        m_shift = m
        for q in np.array(list(range(1))) + m_shift:
            s = subset(self.db, "field", [field])  # pick  field
            maxZ = min(s.maxN)
            modelist = []
            [modelist.append([q, p + 1]) for p in range(maxZ - 1)]
        # print q,'in plotgamma'
        s = subset(s.db, "mn", modelist)

        # set x-range
        xrange = old_div(s.nx, 2) - 2
        xrange = [old_div(s.nx, 2), old_div(s.nx, 2) + xrange]

        # pull up the data
        y = np.array(ListDictKey(s.db, comp))
        print("y.shape", y.shape)

        # in case multiple timesteps are indicated
        all_y = []
        all_yerr = []
        if comp == "amp":
            for elem in t:
                all_y.append(np.squeeze(y[:, elem, :]))
                all_yerr.append(np.squeeze(0 * y[:, elem, :]))
            ynorm = np.max(all_y)
            # all_y = np.array(np.squeeze(all_y))
            # all_yerr = np.array(np.squeeze(all_yerr))

        else:
            all_y.append(np.squeeze(y[:, 0, :]))
            all_yerr.append(np.squeeze(y[:, 1, :]))
            ynorm = s.meta["w_Ln"][0]

        k = s.k  ##nmodes x 2 x nx ndarray k_zeta

        kfactor = np.mean(
            old_div(s.k_r[:, 1, old_div(s.nx, 2)], s.k[:, 1, old_div(s.nx, 2)])
        )  # good enough for now

        for elem in range(np.size(t)):
            # print 'printing line' , elem
            errorline = parhandles.append(
                canvas.errorbar(
                    k[:, 1, old_div(s.nx, 2)],
                    all_y[elem][:, old_div(s.nx, 2)],
                    yerr=all_yerr[elem][:, old_div(s.nx, 2)],
                    fmt=colors[q],
                )
            )

        parlabels.append("m " + str(q))

        # loop over dz sets and connect with dotted line  . . .
        jj = 0  # will reference dz color

        ymin_data = np.max(np.array(ListDictKey(s.db, comp)))
        ymax_data = 0  # for bookeeping

        for p in list(set(s.path).union()):
            sub_s = subset(s.db, "path", [p])
            j = sub_s.dz[0]
            # print sub_s.amp.shape
            s_i = np.argsort(sub_s.mn[:, 1])  # sort by 'local' m, global m is ok also
            # print s_i, sub_s.mn, sub_s.nx, jj
            y = np.array(ListDictKey(sub_s.db, comp))
            y_alt = 2.0 * np.array(ListDictKey(sub_s.db, comp))
            all_y = []
            all_yerr = []
            if comp == "amp":
                for elem in t:
                    all_y.append(np.squeeze(y[:, elem, :]))
                    all_yerr.append(np.squeeze(0 * y[:, elem, :]))
            else:
                all_y.append(np.squeeze(y[:, 0, :]))
                all_yerr.append(np.squeeze(y[:, 1, :]))

            k = sub_s.k  ##

            for elem in range(np.size(t)):
                if q == m_shift:  # fix the parallel mode
                    dzhandles.append(
                        canvas.plot(
                            k[s_i, 1, old_div(sub_s.nx, 2)],
                            all_y[elem][s_i, old_div(sub_s.nx, 2)],
                            color=colordash[jj],
                            alpha=0.5,
                        )
                    )

                    ymin_data = np.min(
                        [np.min(y[s_i, old_div(sub_s.nx, 2)]), ymin_data]
                    )
                    ymax_data = np.max(
                        [np.max(y[s_i, old_div(sub_s.nx, 2)]), ymax_data]
                    )

                    dzlabels.append(j)

                    if yscale == "log":
                        factor = 10
                    else:
                        factor = 2
                    # print 'annotating'
                    canvas.annotate(
                        str(j),
                        (
                            k[s_i[0], 1, old_div(sub_s.nx, 2)],
                            y[elem][s_i[0], old_div(sub_s.nx, 2)],
                        ),
                        fontsize=8,
                    )
                    # p = canvas.axvspan(k[s_i[0],1,sub_s.nx/2], k[s_i[-1],1,sub_s.nx/2],
                    #               facecolor=colordash[jj], alpha=0.01)
                    print("done annotating")
                else:
                    canvas.plot(
                        k[s_i, 1, old_div(sub_s.nx, 2)],
                        y[elem][s_i, old_div(sub_s.nx, 2)],
                        color=colordash[jj],
                        alpha=0.3,
                    )

                jj = jj + 1

        dzhandles = np.array(dzhandles).flatten()
        dzlabels = np.array(dzlabels).flatten()

        dzlabels = list(set(dzlabels).union())

        dz_i = np.argsort(dzlabels)

        dzhandles = dzhandles[dz_i]
        dzlabels_cp = np.array(dzlabels)[dz_i]

        # print type(dzlabels), np.size(dzlabels)
        for i in range(np.size(dzlabels)):
            dzlabels[i] = "DZ: " + str(dzlabels_cp[i])  # +r"$\pi$"

        parlabels = np.array(parlabels).flatten()

        if overplot == True:
            try:
                self.plottheory(pp, canvas=canvas, comp=comp, field=field)
                # self.plottheory(pp,comp=comp)
            except:
                print("no theory plot")
        if infobox:
            textstr = "$\L_{\parallel}=%.2f$\n$\L_{\partial_r n}=%.2f$\n$B=%.2f$" % (
                s.meta["lpar"][old_div(s.nx, 2)],
                s.meta["L"][old_div(s.nx, 2), old_div(s.ny, 2)],
                s.meta["Bpxy"]["v"][old_div(s.nx, 2), old_div(s.ny, 2)],
            )
            props = dict(boxstyle="square", facecolor="white", alpha=0.3)
            textbox = canvas.text(
                0.82,
                0.95,
                textstr,
                transform=canvas.transAxes,
                fontsize=10,
                verticalalignment="top",
                bbox=props,
            )
            # leg = canvas.legend(handles,labels,ncol=2,loc='best',prop={'size':4},fancybox=True)

        # cloney.set_xlim(xmin,xmax)
        try:
            canvas.set_yscale(yscale)
            canvas.set_xscale(xscale)

            if yscale == "symlog":
                canvas.set_yscale(yscale, linthreshy=1e-13)
            if xscale == "symlog":
                canvas.set_xscale(xscale, linthreshy=1e-13)

            if gridON:
                canvas.grid()
        except:
            try:
                canvas.set_yscale("symlog")
            except:
                print("scaling failed completely")

        ##################################################################

        if ownpage and rootfig is None:
            clonex.set_yscale(yscale)  # must be called before limits are set

            try:
                if yscale == "linear":
                    formatter = ticker.ScalarFormatter()
                    formatter.set_powerlimits((-2, 2))  # force scientific notation
                    canvas.yaxis.set_major_formatter(formatter)
                    clonex.yaxis.set_major_formatter(formatter)
                # canvas.useOffset=False
            except:
                print("fail 1")
            [xmin, xmax, ymin, ymax] = canvas.axis()

            if yscale == "symlog":
                clonex.set_yscale(yscale, linthreshy=1e-9)
            if xscale == "symlog":
                clonex.set_xscale(xscale, linthreshy=1e-9)
            # if np.any(s.trans) and trans:
            [xmin1, xmax1, ymin1, ymax1] = canvas.axis()
            if trans:
                try:
                    cloney = canvas.twiny()
                    # cloney.set_yscale(yscale)
                    cloney.set_xscale(xscale)
                    [xmin1, xmax1, ymin2, ymax2] = canvas.axis()

                    if xscale == "symlog":
                        cloney.set_xscale(xscale, linthreshy=1e-9)
                    if yscale == "symlog":
                        cloney.set_yscale(yscale, linthreshy=1e-9)
                    if yscale == "linear":
                        cloney.yaxis.set_major_formatter(formatter)
                except:
                    print("fail trans")

            Ln_drive_scale = s.meta["w_Ln"][0] ** -1
            # Ln_drive_scale = 2.1e3
            # clonex.set_ylim(Ln_drive_scale*ymin, Ln_drive_scale*ymax)
            clonex.set_ylim(ynorm**-1 * ymin, ynorm**-1 * ymax)

            try:
                if trans:
                    # k_factor = #scales from k_zeta to k_perp
                    cloney.set_xlim(kfactor * xmin, kfactor * xmax)

                    cloney.set_ylim(
                        ymin, ymax
                    )  # because cloney shares the yaxis with canvas it may overide them, this fixes that
                    cloney.set_xlabel(r"$k_{\perp} \rho_{ci}$", fontsize=18)
            except:
                print("moar fail")
            # clonex.set_xscale(xscale)

            # ion_acoust_str = r"$\frac{c_s}{L_{\partial_r n}}}$"

            if comp == "gamma":
                canvas.set_ylabel(
                    r"$\frac{\gamma}{\omega_{ci}}$", fontsize=18, rotation="horizontal"
                )
                clonex.set_ylabel(
                    r"$\frac{\gamma}{\frac{c_s}{L_n}}$",
                    color="k",
                    fontsize=18,
                    rotation="horizontal",
                )
            if comp == "freq":
                canvas.set_ylabel(
                    r"$\frac{\omega}{\omega_{ci}}$", fontsize=18, rotation="horizontal"
                )
                clonex.set_ylabel(
                    r"$\frac{\omega}{\frac{c_s}{L_n}}$",
                    color="k",
                    fontsize=18,
                    rotation="horizontal",
                )

            if comp == "amp":
                canvas.set_ylabel(r"$A_k$", fontsize=18, rotation="horizontal")
                clonex.set_ylabel(
                    r"$\frac{A_k}{A_{max}}$",
                    color="k",
                    fontsize=18,
                    rotation="horizontal",
                )

            canvas.set_xlabel(r"$k_{\zeta} \rho_{ci}$", fontsize=18)

            title = comp + " computed from " + field
            # canvas.set_title(title,fontsize=14)
            fig1.suptitle(title, fontsize=14)

        if not ownpage:
            print("probably for a movie")
            fig1 = rootfig
            # canvasjunk = fig1.add_subplot(1,1,1)
            # canvasjunk = canvas

        if save:
            if file is None:
                try:
                    fig1.savefig(pp, format="pdf")
                except:
                    print("pyplt doesnt like you")
            else:
                try:
                    fig1.savefig(file, dpi=200)
                except:
                    print("no movie for you ;(")

        if ownpage:
            # fig1.close()
            plt.close(fig1)

    def plotmodes(
        self,
        pp,
        field="Ni",
        comp="amp",
        math="1",
        ylim=1,
        yscale="symlog",
        clip=False,
        xaxis="t",
        xscale="linear",
        xrange=1,
        debug=False,
        yaxis=r"$\frac{Ni}{Ni_0}$",
        linestyle="-",
        summary=True,
    ):
        Nplots = self.nrun

        colors = ["b", "g", "r", "c", "m", "y", "k", "b", "g", "r", "c", "m", "y", "k"]
        styles = ["^", "s"]

        fig1 = plt.figure()

        fig2 = plt.figure()

        Modes = subset(self.db, "field", [field])  # pick  field

        adj = fig2.subplots_adjust(hspace=0.4, wspace=0.4)
        fig2.suptitle("Dominant mode " + comp + " for  " + field)
        props = dict(alpha=0.8, edgecolors="none")

        allcurves = fig1.add_subplot(1, 1, 1)
        fig1.suptitle("Dominant mode behavior for  " + field)

        modenames = []
        k = 0

        for j in list(set(Modes.path).union()):  #
            s = subset(Modes.db, "path", [j])  # pick run
            dz = s.dz[0]
            xr = list(
                range(
                    old_div(s.nx, 2) - old_div(xrange, 2),
                    old_div(s.nx, 2) + old_div(xrange, 2) + 1,
                )
            )
            data = np.array(
                ListDictKey(s.db, comp)
            )  # pick component should be ok for a fixed dz key

            data = data  # + 1e-32 #hacky way to deal with buggy scaling
            ax = fig2.add_subplot(round(old_div(Nplots, 3.0) + 1.0), 3, k + 1)

            ax.grid(True, linestyle="-", color=".75")
            handles = []
            # modenames.append(str(j))

            # find the "biggest" mode for this dz
            d = data[:, s.nt[0] - 1, :]  # nmode X nx array
            # d = s.gamma[:,2,:]
            where = d == np.nanmax(d)
            z = where.nonzero()  # mode index and n index
            imax = z[0][0]
            # xi_max = z[1][0]
            xi_max = old_div(s.nx, 2)

            if debug and yscale == "log":
                gamma = np.array(ListDictKey(s.db, "gamma"))  # nmodes x 2 x nx

            for i in range(s.nmodes):
                if math == "gamma":
                    out = old_div(np.gradient(data[i, :, xr])[1], data[i, :, xr])
                else:
                    out = data[i, 2:, xi_max]  # skip the first 2 points

                if xaxis == "t":
                    # print 'out.size', out.size, out.shape
                    x = np.array(list(range(out.size)))
                    # plt.plot(x,out.flatten(),c=colors[k])
                    label = str(s.mn[i])
                    # handles.append(ax.plot(x,out.flatten(),
                    #                 c=cm.jet(1.*k),label = label))
                    ax.plot(
                        x, out.flatten(), c=cm.jet(0.2 * i), label=label, linestyle="-"
                    )

                else:
                    x = np.array(ListDictKey(s.db, xaxis))[i, :, xr]
                    # x #an N? by nx array
                    print(x[:, 1], out[:, 0])
                    plt.scatter(x[:, 1], out[:, 0])  # ,c=colors[k])
                    ax.scatter(
                        x[:, 1], out[:, 0]
                    )  # ,c=colors[k])#,alpha = (1 +i)/s.nmodes)

                    # detect error (bar data
                    print("error bars:", x, out)

            # ax.legend(handles,labels,loc='best',prop={'size':6})

            formatter = ticker.ScalarFormatter()
            formatter.set_powerlimits((0, 0))
            ax.xaxis.set_major_formatter(formatter)
            # ax.axis('tight')
            if yscale == "linear":
                ax.yaxis.set_major_formatter(formatter)
            if yscale == "symlog":
                ax.set_yscale("symlog", linthreshy=1e-13)
            else:
                try:
                    ax.set_yscale(yscale)
                except:
                    print("may get weird axis")
                    ax.set_yscale("symlog")
            # if comp=='phase' or yscale=='linear':
            #         ax.set_xscale('symlog',linthreshx=1.0)

            ax.set_xscale(xscale)

            ax.axis("tight")
            artist.setp(ax.axes.get_xticklabels(), fontsize=6)
            artist.setp(ax.axes.get_yticklabels(), fontsize=8)
            # artist.setp(ax.axes.get_yscale(), fontsize=8)
            ax.set_title(str(dz), fontsize=10)
            ax.set_xlabel(xaxis)
            handles, labels = ax.get_legend_handles_labels()
            leg = ax.legend(
                handles, labels, ncol=2, loc="best", prop={"size": 4}, fancybox=True
            )
            leg.get_frame().set_alpha(0.3)
            # x = s.Rxy[imax,:,s.ny/2]

            t0 = 2
            if clip == True:
                t0 = round(old_div(s.nt[0], 3))
            y = np.squeeze(data[imax, t0:, xi_max])
            x = np.array(list(range(y.size)))

            print(imax, xi_max)

            label = (
                str([round(elem, 3) for elem in s.MN[imax]])
                + str(s.mn[imax])
                + " at x= "
                + str(xi_max)
                + " ,"
                + str(
                    round(
                        old_div(s.gamma[imax, 2, xi_max], s.gamma[imax, 0, xi_max]), 3
                    )
                )
                + "%  "
                + str(round(s.gamma[imax, 0, xi_max], 4))
            )

            short_label = str(dz)
            print(short_label, x.shape, y.shape)
            allcurves.plot(x, y, ".", c=cm.jet(1.0 * k / len(x)), label=label)
            # print len(x), k*len(x)/(Nplots+2),s.nrun
            allcurves.annotate(
                short_label,
                (x[k * len(x) / (Nplots + 1)], y[k * len(x) / (Nplots + 1)]),
                fontsize=8,
            )

            # modenames.append(str([round(elem,3) for elem in s.MN[imax]])
            #        +str(s.mn[imax])+' at x= '+str(xi_max)+' ,'+str(s.gamma[imax,2,xi_max]))

            if debug and yscale == "log":
                gam = gamma[imax, 0, xi_max]
                f0 = gamma[imax, 1, xi_max]
                allcurves.plot(x, f0 * np.exp(gam * s.dt[imax] * x), "k:")

            k += 1

        # if ylim:
        #    allcurves.set_ylim(data[,xi_max].min(),5*data[:,xi_max].max())

        fig2.savefig(pp, format="pdf")

        handles, labels = allcurves.get_legend_handles_labels()
        allcurves.legend(handles, labels, loc="best", prop={"size": 6})
        # allcurves.legend(modenames,loc='best',prop={'size':6})
        allcurves.set_title(
            field + " " + comp + ", all runs, " + yscale + " yscale", fontsize=10
        )
        allcurves.set_ylabel(yaxis)
        allcurves.set_xlabel(xaxis)

        if yscale == "linear":
            allcurves.yaxis.set_major_formatter(formatter)
        else:
            try:
                allcurves.set_yscale(yscale)
            except:
                print("may get weird axis scaling")
            if yscale == "log":
                allcurves.axis("tight")
            # allcurves.set_ylim(data.min(),data.max())
        # allcurves.set_yscale(yscale,nonposy='mask')

        # plt.xscale(xscale)

        # plt.legend(modenames,loc='best')
        if summary:
            fig1.savefig(pp, format="pdf")
            plt.close(fig1)

        plt.close(fig2)

    # except:
    #   print "Sorry you fail"

    def plotradeigen(
        self, pp, field="Ni", comp="amp", yscale="linear", xscale="linear"
    ):
        Nplots = self.nrun
        colors = ["b", "g", "r", "c", "m", "y", "k", "b", "g", "r", "c", "m", "y", "k"]
        fig1 = plt.figure()

        fig2 = plt.figure()
        adj = fig2.subplots_adjust(hspace=0.4, wspace=0.4)

        # canvas = FigureCanvas(fig)

        Modes = subset(self.db, "field", [field])

        k = 0
        fig2.suptitle("Dominant mode behavior for  " + field)
        props = dict(alpha=0.8, edgecolors="none")

        allcurves = fig1.add_subplot(1, 1, 1)
        fig1.suptitle("Dominant mode behavior for  " + field)

        modeleg = []

        for p in list(set(Modes.path).union()):
            print(p)
            s = subset(Modes.db, "path", [p])  # pick run
            # data = np.array(ListDictKey(s.db,comp)) #pick component
            j = s.dz[0]
            ax = fig2.add_subplot(round(old_div(Nplots, 3.0) + 1.0), 3, k + 1)
            ax.grid(True, linestyle="-", color=".75")
            data = np.array(ListDictKey(s.db, comp))  # pick component
            handles = []

            # find the "biggest" mode for this dz
            d = data[:, s.nt[0] - 1, :]  # nmode X nx array
            where = d == d.max()
            z = where.nonzero()  # mode index and n index
            imax = z[0][0]
            modeleg.append(
                str([round(elem, 3) for elem in s.MN[imax]]) + str(s.mn[imax])
            )

            # str(s.k[:,1,:][z])

            for i in range(s.nmodes):
                # out = mode[mode.ny/2,:]
                # print i,s.Rxynorm.shape,s.ny
                x = np.squeeze(s.Rxynorm[i, :, old_div(s.ny, 2)])
                y = data[i, s.nt[0] - 1, :]

                handles.append(ax.plot(x, y, c=cm.jet(1.0 * k / len(x))))

            formatter = ticker.ScalarFormatter()
            formatter.set_powerlimits((0, 0))
            ax.xaxis.set_major_formatter(formatter)

            if yscale == "linear":
                ax.yaxis.set_major_formatter(formatter)
            else:
                ax.set_yscale(yscale)

            artist.setp(ax.axes.get_xticklabels(), fontsize=6)
            artist.setp(ax.axes.get_yticklabels(), fontsize=8)
            # artist.setp(ax.axes.get_yscale(), fontsize=8)
            ax.set_title(str(j), fontsize=10)

            x = np.squeeze(s.Rxynorm[imax, :, old_div(s.ny, 2)])
            y = data[imax, s.nt[0] - 1, :]
            # allcurves.plot(x,y,c= colors[k])
            allcurves.plot(x, y, c=cm.jet(0.1 * k / len(x)))
            print(k)
            k = k + 1

        fig2.savefig(pp, format="pdf")
        if yscale == "linear":
            allcurves.yaxis.set_major_formatter(formatter)
        else:
            allcurves.set_yscale(yscale)
        try:
            allcurves.set_xscale(xscale)
        except:
            allcurves.set_xscale("symlog")

        # allcurves.xaxis.set_major_formatter(ticker.NullFormatter())
        allcurves.legend(modeleg, loc="best", prop={"size": 6})
        allcurves.set_xlabel(r"$\frac{x}{\rho_{ci}}$")
        allcurves.set_ylabel(r"$\frac{Ni}{Ni_0}$")
        fig1.savefig(pp, format="pdf")
        plt.close(fig1)
        plt.close(fig2)

    def plotmodes2(
        self,
        pp,
        field="Ni",
        comp="amp",
        math="1",
        ylim=1,
        yscale="symlog",
        clip=0,
        xaxis="t",
        xscale="linear",
        xrange=1,
        debug=False,
    ):
        Nplots = self.nrun
        Modes = subset(self.db, "field", [field])  # pick  field
        colors = ["b", "g", "r", "c", "m", "y", "k", "b", "g", "r", "c", "m", "y", "k"]

        fig = Figure()
        plt.figure()

        canvas = FigureCanvas(fig)
        k = 0
        nrow = round(old_div(Nplots, 3.0) + 1.0)
        ncol = 3
        # nrow = round(Nplots/3.0 + 1.0)
        # ncol = round(Nplots/3.0 + 1.0)
        f, axarr = plt.subplots(int(nrow), int(ncol))

        for p in list(set(Modes.path).union()):  #
            s = subset(Modes.db, "path", [p])  # pick run
            j = s.dz[0]
            xr = list(
                range(
                    old_div(s.nx, 2) - old_div(xrange, 2),
                    old_div(s.nx, 2) + old_div(xrange, 2) + 1,
                )
            )
            data = np.array(ListDictKey(s.db, comp))  # pick component
            # ax =fig.add_subplot(round(Nplots/3.0 + 1.0),3,k+1)

            for i in range(s.nmodes):
                out = data[i, :, xr]

            print(j, i)
            if xaxis == "t":
                x = list(range(out.size))
                # plt.scatter(x,out.flatten(),c=colors[k])
                plt.scatter(x, out.flatten(), c=cm.jet(1.0 * k / len(x)))
                # axarr[j%(ncol),j/ncol].scatter(x,out.flatten(),c=colors[k])#,alpha = (1 +i)/s.nmodes)
                axarr[old_div(j, ncol), j % (ncol)].scatter(
                    x, out.flatten(), c=cm.jet(1.0 * k / len(x))
                )  #

            else:
                x = np.array(ListDictKey(s.db, xaxis))[i, :, xr]
                # x #an N? by nx array
                print(x[:, 1], out[:, 0])
                plt.scatter(x[:, 1], out[:, 0])  # ,c=colors[k])
                axarr[j % (col), old_div(j, col)].scatter(
                    x[:, 1], out[:, 0]
                )  # ,c=colors[k])#,alpha = (1 +i)/s.nmodes)

                # detect error (bar data
                print("error bars:", x, out)

            axarr[old_div(j, ncol), j % (ncol)].set_yscale(yscale)
            axarr[old_div(j, ncol), j % (ncol)].set_xscale(xscale)
            axarr[old_div(j, ncol), j % (ncol)].set_title(str(j), fontsize=10)
            axarr[old_div(j, ncol), j % (ncol)].set_xlabel(xaxis)

            plt.setp([a.get_xticklabels() for a in axarr[0, :]], visible=False)
            plt.setp([a.get_yticklabels() for a in axarr[:, ncol - 1]], visible=False)

            if ylim:
                axarr[j % (ncol), old_div(j, ncol)].set_ylim(data.min(), 5 * data.max())
            k += 1

        plt.title(field + " " + comp + ", all runs, " + yscale + " yscale", fontsize=10)
        plt.xlabel(xaxis)
        if ylim:
            plt.ylim(data.min(), 10 * data.max())

        plt.yscale(yscale, nonposy="mask")
        plt.xscale(xscale)

        fig.savefig(pp, format="pdf")
        plt.savefig(pp, format="pdf")

        plt.close()

        return 0

    def plotMacroDep(
        self,
        pp,
        field="Ni",
        yscale="symlog",
        clip=0,
        xaxis="t",
        xscale="linear",
        xrange=1,
    ):
        colors = ["b", "g", "r", "c", "m", "y", "k", "b", "g", "r", "c", "m", "y", "k"]
        plt.figure()

    def savemovie(
        self, field="Ni", yscale="log", xscale="log", moviename="spectrum.avi"
    ):
        print("Making movie animation.mpg - this make take a while")
        files = []

        for t in range(self.nt[0] - 3):
            print(t)
            filename = str("%03d" % (t + 1) + ".png")
            self.plotvsK(
                "dont need pp",
                yscale="log",
                t=[1, t + 2],
                xscale="log",
                overplot=False,
                comp="amp",
                trans=True,
                file=filename,
            )
            files.append(filename)

        command = (
            "mencoder",
            "mf://*.png",
            "-mf",
            "type=png:w=800:h=600:fps=10",
            "-ovc",
            "lavc",
            "-lavcopts",
            "vcodec=mpeg4",
            "-oac",
            "copy",
            "-o",
            moviename,
        )

        import subprocess, os

        subprocess.check_call(command)
        os.system("rm *png")

    def printmeta(self, pp, filename="output2.pdf", debug=False):
        import os
        from pyPdf import PdfFileWriter, PdfFileReader

        PAGE_HEIGHT = defaultPageSize[1]
        styles = getSampleStyleSheet()
        Title = "BOUT++ Results"
        Author = "Dmitry Meyerson"
        URL = ""
        email = "dmitry.meyerson@gmail.com"
        Abstract = """This document highlights some results from BOUT++ simulation"""
        Elements = []
        HeaderStyle = styles["Heading1"]
        ParaStyle = styles["Normal"]
        PreStyle = styles["Code"]

        def header(
            txt, style=HeaderStyle, klass=Paragraph, sep=0.3
        ):  # return styled text with a space
            s = Spacer(0.2 * inch, sep * inch)
            para = klass(txt, style)
            sect = [s, para]
            result = KeepTogether(sect)
            return result

        def p(txt):  # wrapper for header
            return header(txt, style=ParaStyle, sep=0.1)

        def pre(txt):  # return styled text with a space
            s = Spacer(0.1 * inch, 0.1 * inch)
            p = Preformatted(txt, PreStyle)
            precomps = [s, p]
            result = KeepTogether(precomps)
            return result

        def graphout(name, datain, xaxis=None):
            if xaxis is None:
                xaxis = list(range(datain.size))
            if xlabel is None:
                xlabel = ""
            if ylabel is None:
                ylabel = ""

            drawing = Drawing(400, 200)
            # data = [
            #    ((1,1), (2,2), (2.5,1), (3,3), (4,5)),
            #    ((1,2), (2,3), (2.5,2), (3.5,5), (4,6))
            #    ]
            dataview = [tuple([(xaxis[i], datain[i]) for i in range(datain.size)])]
            lp = LinePlot()
            lp.x = 50
            lp.y = 50
            lp.height = 125
            lp.width = 300
            lp.data = dataview
            lp.xValueAxis.xLabelFormat = "{mmm} {yy}"
            lp.lineLabels.fontSize = 6
            lp.lineLabels.boxStrokeWidth = 0.5
            lp.lineLabels.visible = 1
            lp.lineLabels.boxAnchor = "c"
            # lp.joinedLines = 1
            # lp.lines[0].symbol = makeMarker('FilledCircle')
            # lp.lines[1].symbol = makeMarker('Circle')
            # lp.lineLabelFormat = '%2.0f'
            # lp.strokeColor = colors.black
            # lp.xValueAxis.valueMin = min(xaxis)
            # lp.xValueAxis.valueMax = max(xaxis)
            # lp.xValueAxis.valueSteps = xaxis
            # lp.xValueAxis.labelTextFormat = '%2.1f'
            # lp.yValueAxis.valueMin = min(datain)
            # lp.yValueAxis.valueMax = max(datain)
            # lp.yValueAxis.valueSteps = [1, 2, 3, 5, 6]
            drawing.add(lp)
            return drawing

        def go():
            doc = SimpleDocTemplate("meta.pdf")
            doc.build(Elements)

        mytitle = header(Title)
        myname = header(Author, sep=0.1, style=ParaStyle)
        mysite = header(URL, sep=0.1, style=ParaStyle)
        mymail = header(email, sep=0.1, style=ParaStyle)
        abstract_title = header("ABSTRACT")
        myabstract = p(Abstract)
        head_info = [mytitle, myname, mysite, mymail, abstract_title, myabstract]
        Elements.extend(head_info)

        meta_title = header("metadata", sep=0)
        metasection = []
        metasection.append(meta_title)

        for i, elem in enumerate(self.meta):
            # if type(self.meta[elem]) != type(np.array([])):

            print(elem, type(self.meta[elem]))

            if type(self.meta[elem]) == type({}):
                print("{}")
                data = np.array(self.meta[elem]["v"])
                unit_label = str(self.meta[elem]["u"])
            else:
                data = np.array(self.meta[elem])
                unit_label = ""

            xaxis = np.squeeze(self.meta["Rxy"]["v"][:, old_div(self.ny, 2)])

            if data.shape == (self.nx, self.ny):
                datastr = np.squeeze(data[:, old_div(self.ny, 2)])
                # metasection.append(graphout('stuff',datastr,xaxis=xaxis))
                # metasection.append(RL_Plot(datastr,xaxis))

                metasection.append(RL_Plot(datastr, xaxis, linelabel=str(elem)))
            # metasection.append(RL_Plot(datastr,xaxis,xlabel='xlabel'))
            elif data.shape == self.nx:
                datastr = data
            # metasection.append(graphout('stuff',datastr,xaxis=xaxis))
            # metasection.append(RL_Plot(datastr,xaxis,linelabel=str(elem)))
            elif data.shape == (1,):
                data = data[0]
                metasection.append(
                    header(
                        str(elem) + ": " + str(data) + " " + unit_label,
                        sep=0.1,
                        style=ParaStyle,
                    )
                )
            else:
                print(elem, data, data.shape)
                metasection.append(
                    header(
                        str(elem) + ": " + str(data) + " " + unit_label,
                        sep=0.1,
                        style=ParaStyle,
                    )
                )

        src = KeepTogether(metasection)
        Elements.append(src)

        cxxtitle = header("Equations in CXX")
        cxxsection = []
        # print self.cxx
        cxxsection.append(header(self.cxx[0], sep=0.1, style=ParaStyle))
        cxxsrc = KeepTogether(cxxsection)

        Elements.append(cxxsrc)
        # for i,elem in enumerate(self.cxx):
        #    if type(self.meta[elem])== type({}):
        #       print elem #np.array(self.meta[elem]['v']).shape()
        #       if np.array(self.meta[elem]['v']).shape == (self.nx,self.ny):
        #          datastr = str(self.meta[elem]['v'][:,self.ny/2])
        #          metasection.append(graphout('stuff',
        #                                      self.meta[elem]['v'][:,self.ny/2]))
        #       else:
        #          datastr = str(self.meta[elem]['v'])
        #       metasection.append(header(str(elem)+': '+datastr
        #                                    + ' '+ str(self.meta[elem]['u']),
        #                                    sep=0.1, style=ParaStyle))

        if debug:
            return Elements
        go()

        output = PdfFileWriter()
        metapdf = PdfFileReader(file("meta.pdf", "rb"))
        mainpdf = PdfFileReader(file("output.pdf", "rb"))

        for i in range(0, metapdf.getNumPages()):
            output.addPage(metapdf.getPage(i))

        for i in range(0, mainpdf.getNumPages()):
            output.addPage(mainpdf.getPage(i))

        outputFile = filename
        outputStream = file(outputFile, "wb")
        output.write(outputStream)
        outputStream.close()
        print("Consolidation complete.")


class subset(LinResDraw):
    def __init__(self, alldb, key, valuelist, model=False):
        selection = ListDictFilt(alldb, key, valuelist)
        if len(selection) != 0:
            LinRes.__init__(self, selection)
            self.skey = key
            if model == True:
                self.model()
        else:
            LinRes.__init__(self, alldb)
            if model == True:
                self.model()



#!/bin/python3
# -*- coding: utf-8

#!##################################################################
#
#       2D blob simulations
#
#  Copyright:   NR Walkden, B Dudson, D Schwörer;  2012, , 2017
#
###################################################################

import boutpp as bc
from numpy import sqrt
from boutpp import bracket, DDZ, Delp2
import sys

bc.init("-d blob".split(" ") + sys.argv[1:])
mesh = bc.Mesh.getGlobal()
n = bc.Field3D.fromMesh(mesh)
omega = bc.Field3D.fromMesh(mesh)
phi = bc.Field3D.fromMesh(mesh)

phiSolver = bc.Laplacian()

options = bc.Options("model")
# Temperature in eV
Te0 = options.get("Te0", 30)
e = options.get("e", 1.602e-19)
m_i = options.get("m_i", 2 * 1.667e-27)
m_e = options.get("m_e", 9.11e-31)

# Background density in cubic m
n0 = options.get("n0", 1e19)
# Viscous diffusion coefficient
D_vort = options.get("D_vort", 0)
# Density diffusion coefficient
D_n = options.get("D_n", 0)

# Radius of curvature [m]
R_c = options.get("R_c",   1.5)
# Parallel connection length [m]
L_par = options.get("L_par", 10)

# Value of magnetic field strength [T]
B0 = options.get("B0", 0.35)


# System option switches

# Include compressible ExB term in density equation
compressible = options.get("compressible", False)
# Use Boussinesq approximation in vorticity
boussinesq = options.get("boussinesq", True)
# Sheath closure
sheath = options.get("sheath", True)


Omega_i = e * B0 / m_i           # Cyclotron Frequency
c_s = sqrt(e * Te0 / m_i)      # Bohm sound speed
rho_s = c_s / Omega_i          # Bohm gyro-radius

print("\n\n\t----------Parameters: ------------ \n\tOmega_i = %e /s,\n\t"
      "c_s = %e m/s,\n\trho_s = %e m\n" % (Omega_i, c_s, rho_s))

# Calculate delta_*, blob size scaling
print("\tdelta_* = rho_s * (dn/n) * %e "
      % (pow(L_par * L_par / (R_c * rho_s), 1. / 5)))

# /************ Create a solver for potential ********/

if boussinesq:
    # BOUT.inp section "phiBoussinesq"
    phiSolver = bc.Laplacian(bc.Options("phiBoussinesq"))
else:
        # BOUT.inp section "phiSolver"
    phiSolver = bc.Laplacian(bc.Options("phiSolver"))

phi = bc.Field3D.fromMesh(mesh)
phi.set(0.0)  # Starting guess for first solve (if iterative)

# /************ Tell BOUT++ what to solve ************/

model = bc.PhysicsModelBase()
model.solve_for(n=n, omega=omega)
# model.save_repeat(phi=phi)
# model.save_once(rho_s=rho_s,c_s=c_s,Omega_i=Omega_i)


def rhs(time):
    global n, omega, phi
    # Run communications
    ######################################
    mesh.communicate(n, omega)

    # Invert div(n grad(phi)) = grad(n) grad(phi) + n Delp_perp^2(phi) = omega
    ######################################
    # Set the time derivative by adding/... to it
    # make sure to never overwrite it
    # ddt_n = bla does NOT set the time derivative
    ddt_n = n.ddt()
    ddt_n.set(0)
    if not boussinesq:
        # Including full density in vorticit inversion
          # Update the 'C' coefficient. See invert_laplace.hxx
        phiSolver.setCoefC(n)
        # Use previous solution as guess
        phi = phiSolver.solve(omega / n, phi)
    else:
        # Background density only (1 in normalised units)
        phi = phiSolver.solve(omega, phi)

    mesh.communicate(phi)

    # Density Evolution
    # /

    ddt_n += -bracket(phi, n, "BRACKET_SIMPLE")    # ExB term
    ddt_n += 2 * DDZ(n) * (rho_s / R_c)               # Curvature term
    ddt_n += D_n * Delp2(n)                       # Diffusion term

    if compressible:
        ddt_n -= 2 * n * DDZ(phi) * (rho_s / R_c)       # ExB Compression term

    if sheath:
        # Sheath closure
        ddt_n += n * phi * (rho_s / L_par)  # - (n - 1)*(rho_s/L_par)

    # Vorticity evolution
    # /
        # ExB term
    ddt_omega = -bracket(phi, omega, "BRACKET_SIMPLE")
    ddt_omega += 2 * DDZ(n) * (rho_s / R_c) / n
    # Viscous diffusion term
    ddt_omega += D_vort * Delp2(omega) / n

    if sheath:
        ddt_omega += phi * (rho_s / L_par)
    # other option to set time derivaitve:
    # create a field and set it in the end
    omega.ddt(ddt_omega)

# Start the simulation
model.setRhs(rhs)
model.solve()



from __future__ import print_function
from __future__ import absolute_import

##################################################
#            BOUT++ data package
#
# Routines for examining simulation results for BOUT++
#
##################################################

print("Loading BOUT++ post processing routines")

# Load routines from separate files
import sys
import os

try:
    boutpath = os.environ["BOUT_TOP"]
    pylibpath = boutpath + "/tools/pylib"
    boutdatapath = pylibpath + "/boutdata"
    boututilpath = pylibpath + "/boututils"
    allpath = [boutpath, pylibpath, boutdatapath, boututilpath]
    [sys.path.append(elem) for elem in allpath]
    print(sys.path)

    # sys.path.append('/home/cryosphere/BOUT/tools/pylib')
    # sys.path.append('/home/cryosphere/BOUT/tools/pylib/boutdata')
    # sys.path.append('/home/cryosphere/BOUT/tools/pylib/boututils')

    print("in post_bout/__init__.py")

    import matplotlib

    matplotlib.use("pdf")  # savemovie must be called as a diff. sesssion

    import gobject
    import numpy as np
except ImportError:
    print("can't find the modules I need, you fail")
    sys.exit()  # no point in going on


# import some bout specific modules
try:
    import boutdata
    import boututils
except:
    print("can't find bout related modules, you fail")

# import some home-brewed modules


# create some aliases


try:
    from read_grid import read_grid
except:
    print("Sorry, no read_grid")

try:
    from .read_inp import parse_inp, read_inp, read_log, metadata
except:
    print("Sorry no parse_inp")

try:
    from .read_cxx import read_cxx, get_evolved_cxx, no_comment_cxx
except:
    print("Sorry no read_cxx")

try:
    from post_bout import save, read
except:
    print("Sorry, no show")

try:
    from .basic_info import basic_info, fft_info
except:
    print("Sorry, no basic_info")

try:
    from .pb_corral import corral, LinRes, subset
except:
    print("No corral")

try:
    from . import ListDict
except:
    print("No ListDict")

try:
    # from rotate_mp import rotate
    from rotate2 import rotate
except:
    print("No rotate")



from __future__ import print_function
import sys
import os

try:
    boutpath = os.environ["BOUT_TOP"]
    pylibpath = boutpath + "/pylib"
    pbpath = pylibpath + "/post_bout"
    boutdatapath = pylibpath + "/boutdata"
    boututilpath = pylibpath + "/boututils"

    allpath = [boutpath, pylibpath, pbpath, boutdatapath, boututilpath]
    [sys.path.append(elem) for elem in allpath]
except:
    print("meh")

import numpy as np


def ListDictKey(input, key):
    # given a key and a list of dictionaries this method returns an ordered
    # list of requested key values
    output = []
    for x in input:
        try:
            # print x[key]
            output.append(x[key])
        except:
            print("Key not found")
            return 1

    return output


def ListDictFilt(input, key, valuelist):
    # given a key,value pair and a list of dictionaries this
    # method returns an ordered list of dictionaries where (dict(key)==value) = True
    # http://stackoverflow.com/questions/5762643/how-to-filter-list-of-dictionaries-with-matching-values-for-a-given-key
    try:
        x = copyf(input, key, valuelist)
        return x
    except:
        return []


def copyf(dictlist, key, valuelist):
    return [dictio for dictio in dictlist if dictio[key] in valuelist]


# def subset(obj):
#     #def __init__(self,alldb,key,valuelist,model=False):
#     selection = ListDictFilt(obj.mode_db,obj.key,valuelist)
#     if len(selection) !=0:
#         LinRes.__init__(obj,selection)
#         self.skey = key
#         if model==True:
#             self.model()
#         else:
#             LinRes.__init__(self,alldb)
#             if model==True:
#                 self.model()



import csv


def read_file(filename):
    reader = csv.reader(open(filename, 'r'), delimiter='\t',
                        skipinitialspace=True)

    # Skip header
    for _, _ in zip(range(4), reader):
        continue

    from collections import OrderedDict
    case_lines = OrderedDict() #{}
    for line in reader:
        if line == []:
            break
        case_lines[line[0].rstrip('.')] = line[1]

    titles = next(reader)
    cases_weak = {col.strip(): [] for col in titles[:-1]}

    for line in reader:
        if line == []:
            break
        for title, col in zip(titles, line[:-1]):
            cases_weak[title].append(float(col))

    axis = cases_weak['Local grid']
    data = [cases_weak[x] for x in case_lines]
    labels = [case_lines[x] for x in case_lines]
    return {'axis':axis, 'data':data, 'labels':labels}

def getScan(baseName = "timing_{n}.txt", nthreads = (1,2,4,8,16,32)):
    from numpy import zeros

    dataS = []
    for n in nthreads:
        f = baseName.format(n=n)
        dataS.append(read_file(f))


    nnt = len(dataS)
    nlines = len(dataS[0]['data'])
    nval = len(dataS[0]['data'][0])
    rawDat = zeros([nnt,nval,nlines])

    for i, dat in enumerate(dataS):
        print(len(dat['data']))
        
        rawDat[i,:,:] = dat['data']

    axes = [nthreads, dataS[0]['axis']]

    return axes, rawDat, dataS[0]['labels']



#!/usr/bin/env python3

import argparse
import copy
import difflib
import re
import textwrap

# List of macros, their replacements and what header to find them
# in. Each element should be a dict with "old", "new" and "headers"
# keys, with "old" and "new" values being strings, and "headers" being a
# list of strings. "new" can also be None if the macro has been removed, which
# will cause an error to be printed if the macro is found.
MACRO_REPLACEMENTS = [
    {
        "old": "REVISION",
        "new": "bout::version::revision",
        "headers": ["bout/revision.hxx"],
        "macro": False,
        "always_defined": True,
    },
    {
        "old": "BOUT_VERSION_DOUBLE",
        "new": "bout::version::as_double",
        "headers": ["bout/version.hxx", "bout.hxx"],
        "macro": False,
        "always_defined": True,
    },
    {
        "old": "BOUT_VERSION_STRING",
        "new": "bout::version::full",
        "headers": ["bout/version.hxx", "bout.hxx"],
        "macro": False,
        "always_defined": True,
    },
    # Next one is not technically a macro, but near enough
    {
        "old": "BOUT_VERSION",
        "new": "bout::version::full",
        "headers": ["bout/version.hxx", "bout.hxx"],
        "macro": False,
        "always_defined": True,
    },
    {
        "old": "BACKTRACE",
        "new": "BOUT_USE_BACKTRACE",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "HAS_ARKODE",
        "new": "BOUT_HAS_ARKODE",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "HAS_CVODE",
        "new": "BOUT_HAS_CVODE",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "HAS_HDF5",
        "new": None,
        "headers": [],
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "HAS_IDA",
        "new": "BOUT_HAS_IDA",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "HAS_LAPACK",
        "new": "BOUT_HAS_LAPACK",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "LAPACK",
        "new": "BOUT_HAS_LAPACK",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "HAS_NETCDF",
        "new": "BOUT_HAS_NETCDF",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "HAS_PETSC",
        "new": "BOUT_HAS_PETSC",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "HAS_PRETTY_FUNCTION",
        "new": "BOUT_HAS_PRETTY_FUNCTION",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "HAS_PVODE",
        "new": "BOUT_HAS_PVODE",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "TRACK",
        "new": "BOUT_USE_TRACK",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "NCDF4",
        "new": "BOUT_HAS_NETCDF",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "NCDF",
        "new": "BOUT_HAS_LEGACY_NETCDF",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "HDF5",
        "new": None,
        "headers": [],
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "DEBUG_ENABLED",
        "new": "BOUT_USE_OUTPUT_DEBUG",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "BOUT_FPE",
        "new": "BOUT_USE_SIGFPE",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "LOGCOLOR",
        "new": "BOUT_USE_COLOR",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
    {
        "old": "OPENMP_SCHEDULE",
        "new": "BOUT_OPENMP_SCHEDULE",
        "headers": "bout/build_config.hxx",
        "macro": True,
        "always_defined": True,
    },
]


def fix_include_version_header(old, headers, source):
    """Make sure version.hxx header is included"""

    if not isinstance(headers, list):
        headers = [headers]

    # If header is already included, we can skip this fix
    for header in headers:
        if (
            re.search(r'^#\s*include.*(<|"){}(>|")'.format(header), source, flags=re.M)
            is not None
        ):
            return source

    # If the old macro isn't in the file, we can skip this fix
    if re.search(r"\b{}\b".format(old), source) is None:
        return source

    # Now we want to find a suitable place to stick the new include
    # Good candidates are includes of BOUT++ headers
    includes = []
    source_lines = source.splitlines()
    for linenumber, line in enumerate(source_lines):
        if re.match(r"^#\s*include.*bout/", line):
            includes.append(linenumber)
        if re.match(r"^#\s*include.*physicsmodel", line):
            includes.append(linenumber)

    if includes:
        last_include = includes[-1] + 1
    else:
        # No suitable includes, so just stick at the top of the file
        last_include = 0
    source_lines.insert(last_include, '#include "{}"'.format(headers[0]))

    return "\n".join(source_lines)


def fix_ifdefs(old, source):
    """Remove any code inside #ifdef/#ifndef blocks that would now not be compiled"""
    source_lines = source.splitlines()

    # Something to keep track of nested sections
    in_ifdef = None
    # List of (#ifdef or #ifndef, dict of start/else/end lines)
    macro_blocks = []
    for linenumber, line in enumerate(source_lines):
        if_def = re.match(r"#\s*(ifn?def)\s*(.*)", line)
        else_block = re.match(r"#\s*else", line)
        endif = re.match(r"#\s*endif", line)
        if not (if_def or else_block or endif):
            continue
        # Now we need to keep track of whether we're inside an
        # interesting #ifdef/ifndef, as they might be nested, and we
        # want to find the matching #endif and #else
        if endif:
            if in_ifdef is not None:
                in_ifdef -= 1
            if in_ifdef == 0:
                in_ifdef = None
                macro_blocks[-1]["end"] = linenumber
            continue
        if else_block:
            if in_ifdef == 1:
                macro_blocks[-1]["else"] = linenumber
            continue
        if if_def.group(2) == old:
            in_ifdef = 1
            macro_blocks.append({"start": linenumber, "if_def_type": if_def.group(1)})
        elif in_ifdef is not None:
            in_ifdef += 1

    if macro_blocks == []:
        return source

    # Get all of the lines to be removed
    lines_to_remove = set()
    for block in macro_blocks:
        lines_to_remove |= set(block.values())
        if block["if_def_type"] == "ifdef":
            if "else" in block:
                # Delete the #else block for #ifdef
                lines_to_remove |= set(range(block["else"], block["end"]))
        else:
            # Keep the #else block for #ifndef if there is one, otherwise remove the
            # whole block
            lines_to_remove |= set(
                range(block["start"], block.get("else", block["end"]))
            )

    # Apparently this is actually the best way of removing a bunch of (possibly)
    # non-contiguous indices
    modified_lines = [
        line for num, line in enumerate(source_lines) if num not in lines_to_remove
    ]

    return "\n".join(modified_lines)


def fix_always_defined_macros(old, new, source):
    """Fix '#ifdef's that should become plain '#if'"""
    new_source = re.sub(r"#ifdef\s+{}\b".format(old), r"#if {}".format(new), source)
    return re.sub(r"#ifndef\s+{}\b".format(old), r"#if !{}".format(new), new_source)


def fix_replacement(old, new, source):
    """Straight replacements"""
    return re.sub(r'([^"_])\b{}\b([^"_])'.format(old), r"\1{}\2".format(new), source)


def apply_fixes(replacements, source):
    """Apply all fixes in this module"""
    modified = copy.deepcopy(source)

    for replacement in replacements:
        if replacement["new"] is None:
            print(
                "'%s' has been removed, please delete from your code"
                % replacement["old"]
            )
            continue

        modified = fix_include_version_header(
            replacement["old"], replacement["headers"], modified
        )
        if replacement["macro"] and replacement["always_defined"]:
            modified = fix_always_defined_macros(
                replacement["old"], replacement["new"], modified
            )
        elif replacement["always_defined"]:
            modified = fix_ifdefs(replacement["old"], modified)
        modified = fix_replacement(replacement["old"], replacement["new"], modified)

    return modified


def yes_or_no(question):
    """Convert user input from yes/no variations to True/False"""
    while True:
        reply = input(question + " [y/N] ").lower().strip()
        if not reply or reply[0] == "n":
            return False
        if reply[0] == "y":
            return True


def create_patch(filename, original, modified):
    """Create a unified diff between original and modified"""

    patch = "\n".join(
        difflib.unified_diff(
            original.splitlines(),
            modified.splitlines(),
            fromfile=filename,
            tofile=filename,
            lineterm="",
        )
    )

    return patch


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=textwrap.dedent(
            """\
            Fix macro defines for BOUT++ v4 -> v5

            Please note that this is only slightly better than dumb text replacement. It
            will fix the following:

            * replacement of macros with variables or new names
            * inclusion of correct headers for new variables
            * removal of #if(n)def/#endif blocks that do simple checks for the old
              macro, keeping the appriopriate part, if replaced by a variable
            * change '#if(n)def' for '#if (!)' if the replacment is always defined

            It will try not to replace quoted macro names, but may
            still replace them in strings or comments.

            Please check the diff output carefully!
            """
        ),
    )

    parser.add_argument("files", action="store", nargs="+", help="Input files")
    parser.add_argument(
        "--force", "-f", action="store_true", help="Make changes without asking"
    )
    parser.add_argument(
        "--quiet", "-q", action="store_true", help="Don't print patches"
    )
    parser.add_argument(
        "--patch-only", "-p", action="store_true", help="Print the patches and exit"
    )

    args = parser.parse_args()

    if args.force and args.patch_only:
        raise ValueError("Incompatible options: --force and --patch")

    for filename in args.files:
        with open(filename, "r") as f:
            contents = f.read()
        original = copy.deepcopy(contents)

        modified = apply_fixes(MACRO_REPLACEMENTS, contents)
        patch = create_patch(filename, original, modified)

        if args.patch_only:
            print(patch)
            continue

        if not patch:
            if not args.quiet:
                print("No changes to make to {}".format(filename))
            continue

        if not args.quiet:
            print("\n******************************************")
            print("Changes to {}\n".format(filename))
            print(patch)
            print("\n******************************************")

        if args.force:
            make_change = True
        else:
            make_change = yes_or_no("Make changes to {}?".format(filename))

        if make_change:
            with open(filename, "w") as f:
                f.write(modified)



from __future__ import print_function
######
# Computes the rms of a variable and prints the growth rate value at the last timestep
######

import numpy as np
from boutdata.collect import collect
from boututils.moment_xyzt import moment_xyzt



path='./data/'


p=collect('P',path=path)
rmsp_f=moment_xyzt(p[:,34:35,32:33,:], 'RMS').rms


print(np.gradient(np.log(rmsp_f[:,0,0]))[-1])



from __future__ import division
from builtins import range
from past.utils import old_div

###
# rms(f) : compute growth rate vs. time based on rms of bout variable f for all grid points
# plot_rms (x,y): plots the graph growth_rate vs. time for grid point x,y
###


import numpy as np
from pylab import plot, show, xlabel, ylabel, tight_layout
from boutdata.collect import collect


def rms(f):
    nt = f.shape[0]

    ns = f.shape[1]
    ne = f.shape[2]
    nz = f.shape[3]

    ar = np.zeros([nz])

    rms = np.zeros([nt, ns, ne])

    for i in range(nt):
        for j in range(ns):
            for k in range(ne):
                ar = f[i, j, k, :]
                valav = np.sum(ar)
                tot = np.sum(old_div(np.power(ar - valav, 2), nz))
                rms[i, j, k] = np.sqrt(tot)
    return rms


def plot_rms(x, y):
    s = plot(np.gradient(np.log(rmsp[:, x, y])))
    ylabel("$\gamma / \omega_A$", fontsize=25)
    xlabel("Time$(\\tau_A)$", fontsize=25)
    tight_layout()
    return s


# test
if __name__ == "__main__":
    path = "../../../examples/elm-pb/data"

    data = collect("P", path=path)

    rmsp = rms(data)

    plot_rms(34, 32)
    tight_layout()
    show()



from __future__ import print_function
from builtins import range

###
# compute average growth rate bout variable f and plane y
# prints value in plane y and total average
# optional tind excludes initial 'tind' time steps
# Note it masks the values != Inf
###
import numpy as np
from boutdata.collect import collect
from boututils.moment_xyzt import moment_xyzt


def avgrate(p, y=None, tind=None):
    if tind is None:
        tind = 0

    rmsp_f = moment_xyzt(p, "RMS").rms

    ni = np.shape(rmsp_f)[1]
    nj = np.shape(rmsp_f)[2]

    growth = np.zeros((ni, nj))

    with np.errstate(divide="ignore"):
        for i in range(ni):
            for j in range(nj):
                growth[i, j] = np.gradient(np.log(rmsp_f[tind::, i, j]))[-1]

    d = np.ma.masked_array(growth, np.isnan(growth))

    # masked arrays
    # http://stackoverflow.com/questions/5480694/numpy-calculate-averages-with-nans-removed

    print("Total average growth rate= ", np.mean(np.ma.masked_array(d, np.isinf(d))))
    if y is not None:
        print(
            "Growth rate in plane",
            y,
            "= ",
            np.mean(np.ma.masked_array(growth[:, y], np.isnan(growth[:, y]))),
        )


# test
if __name__ == "__main__":
    path = "/Users/brey/BOUT/bout/examples/elm-pb/data"

    data = collect("P", path=path)

    avgrate(data, 32)



#!/usr/bin/env python3

from boutdata import collect
from matplotlib import pyplot
from sys import argv, exit

datadir = argv[1]

yind = int(argv[2])

xg = 2

f = collect('f', path=datadir)[xg:-xg, :, :]
rhs = collect('rhs', path=datadir)[xg:-xg, :, :]
rhs_check = collect('rhs_check', path=datadir)[xg:-xg, :, :]
error = collect('error', path=datadir)[xg:-xg, :, :]

pyplot.subplot(221)
pyplot.pcolormesh(f[:, yind, :])
pyplot.colorbar()
pyplot.title('f')
pyplot.subplot(222)
pyplot.pcolormesh(rhs[:, yind, :])
pyplot.colorbar()
pyplot.title('rhs')
pyplot.subplot(223)
pyplot.pcolormesh(rhs_check[:, yind, :])
pyplot.colorbar()
pyplot.title('rhs_check')
pyplot.subplot(224)
pyplot.pcolormesh(error[:, yind, :])
pyplot.colorbar()
pyplot.title('error')

pyplot.show()

exit(0)



#!/usr/bin/env python

# Demonstrates how to generate Poincare plots of magnetic fields

import zoidberg

# Size of the domain in y (periodic)
yperiod = 10.

# Define magnetic field
magnetic_field = zoidberg.field.StraightStellarator(I_coil=0.4, radius = 1.0, yperiod = yperiod)

# Make Poincare plot
zoidberg.plot.plot_poincare(magnetic_field, 
                            np.linspace(0,0.5,5),   # x starting positions
                            0.0,                    # z starting positions
                            yperiod,                # Periodicity of y domain
                            interactive=True)       # Click on plot to add points



from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div

from numpy import *;
#from scipy.io import readsav;
import matplotlib.pyplot as plt;
from boutdata.collect import collect

# Dynamic matplotlib settings
from matplotlib import rcParams;
rcParams['font.size'] = 20;
rcParams['legend.fontsize'] = 'small';
rcParams['legend.labelspacing'] = 0.1;
rcParams['lines.linewidth'] = 2;
rcParams['savefig.bbox'] = 'tight';

# Create image directory if not exists
import os;
if not os.path.exists('image'):
   os.makedirs('image');

path='./data/'
data=collect('P',path=path)

#fphi = transpose(readsav('fphi.idl.dat')['fphi'])[:,:,:,];
fphi = fft.fft(data, axis=3)

plt.figure();
for i in range(1, 9):
   print("Growth rate for mode number", i)
   print(gradient(log(abs(fphi[:,34, 32, i]))))
   plt.semilogy(((abs(fphi[:,34, 32, i]))), label = 'n=' + str(i * 5));

plt.legend(loc=2);
plt.xlabel('Time');
plt.savefig('image/plotmode.png');
plt.savefig('image/plotmode.eps');


plt.show(block=False);
plt.figure();
for i in range(1, 9):
   plt.plot(abs(fphi[-1, :, 32, i]), label = 'n=' + str(i * 5));

plt.legend();
plt.xlabel('X index');

plt.savefig('image/plotmodeamp.png');
plt.savefig('image/plotmodeamp.eps');

plt.show(block=False);

plt.figure();
for i in range(1, 9):
   plt.plot(old_div(abs(fphi[-1, :, 32, i]),abs(fphi[-1, :, 32, i]).max()), label = 'n=' + str(i * 5));

plt.legend();
plt.xlabel('X index');

plt.savefig('image/plotmodenorm.png');
plt.savefig('image/plotmodenorm.eps');

plt.show();



from __future__ import print_function
from __future__ import absolute_import
from builtins import str
from builtins import range
from .pb_draw import LinResDraw, subset
from .pb_corral import LinRes
from .pb_nonlinear import NLinResDraw
from pb_transport import Transport

import numpy as np
import matplotlib.pyplot as plt

import matplotlib.pyplot as plt
from matplotlib.figure import Figure
from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
from matplotlib.backends.backend_pdf import PdfPages
import matplotlib.artist as artist
import matplotlib.ticker as ticker

# from matplotlib.ticker import FuncFormatter
# from matplotlib.ticker import ScalarFormatter

from reportlab.platypus import *
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.rl_config import defaultPageSize
from reportlab.lib.units import inch
from reportlab.graphics.charts.linecharts import HorizontalLineChart
from reportlab.graphics.shapes import Drawing
from reportlab.graphics.charts.lineplots import LinePlot
from reportlab.graphics.widgets.markers import makeMarker
from reportlab.lib import colors

from replab_x_vs_y import RL_Plot

# for movie making
from multiprocessing import Queue, Pool
import multiprocessing
import subprocess

# uses  LinResDraw to make a pdf


class LinResPresent(LinResDraw, NLinResDraw, Transport):
    def __init__(self, alldb):
        LinResDraw.__init__(self, alldb)
        NLinResDraw.__init__(self, alldb)
        Transport.__init__(self, alldb)

    def show(
        self,
        filter=True,
        quick=False,
        pdfname="output2.pdf",
        debug=False,
        spectrum_movie=False,
    ):
        colors = ["b", "g", "r", "c", "m", "y", "k", "b", "g", "r", "c", "m", "y", "k"]
        pp = PdfPages("output.pdf")

        # start by removing modes above the maxN threshold
        modelist = []
        [
            modelist.append(list(self.modeid[p]))
            for p in range(self.nmodes)
            if self.mn[p][1] <= self.maxN[p]
        ]

        s = subset(self.db, "modeid", modelist)

        try:
            # fig = Figure(figsize=(6,6))
            # fig = plt.figure()
            dz0 = list(set(s.dz).union())[0]
            ss = subset(s.db, "dz", [dz0])

            # show initial condition and the first step after
            s.plotvsK(
                pp,
                yscale="log",
                xscale="log",
                t=[0, 1, -1],
                overplot=False,
                comp="amp",
                trans=True,
            )

            if spectrum_movie:
                ss.savemovie()

        except:
            print("no scatter")
        # 2D true NM spectrum with color code and boxes around spectral res regions log scale

        plt.figure()
        i = 0
        for j in list(
            set(s.dz).union()
        ):  # looping over runs, over unique 'dz' key values
            ss = subset(s.db, "dz", [j])  # subset where dz = j
            plt.scatter(ss.MN[:, 1], ss.MN[:, 0], c=colors[i])
            plt.annotate(str(j), (ss.MN[0, 1], ss.MN[0, 0]))
            i += 1

        plt.title(" Ni spectrum at t=0, all x")
        plt.ylabel("M -parallel")
        plt.xlabel("N -  axisymmteric")
        plt.xscale("log")
        plt.grid(True, linestyle="-", color=".75")

        try:
            plt.savefig(pp, format="pdf")
        except:
            print("FAILED TO save 1st part")

        plt.close()

        # for elem in self.meta['evolved']['v']:
        #    s.plotnl(pp

        if self.meta["nonlinear"]["v"] == "true":
            self.plotnlrhs(pp)

        if self.meta["transport"] == "true":
            self.plotnlrms(pp)

        for elem in self.meta["evolved"]:
            s.plotmodes(
                pp,
                yscale="symlog",
                comp="phase",
                linestyle=".",
                field=elem,
                summary=False,
            )
            s.plotmodes(pp, yscale="symlog", field=elem, summary=False)
            print(elem)
            try:
                s.plotmodes(
                    pp, yscale="symlog", field=elem, comp="gamma_i", summary=False
                )
            except:
                print("gamma_i plot for " + elem + " failed")

        # s.plotmodes(pp,yscale='symlog',summary=False)

        modelist = []
        # maxZ =
        # [modelist.append([1,p+1]) for p in range(maxZ-1)]
        [
            modelist.append(list(self.modeid[p]))
            for p in range(self.nmodes)
            if self.mn[p][1] <= self.maxN[p]
        ]
        ss = subset(s.db, "mn", modelist)

        if debug:  # just a few problematic slides
            fig1 = plt.figure()
            pp_bug = PdfPages("debug.pdf")
            # ss.plotmodes(pp_bug,yscale='symlog',comp='phase',summary=False)
            s.plotfreq2(pp_bug, xscale="log", yscale="symlog", overplot=True)
            ss.plotgamma(pp_bug, xscale="log", yscale="symlog", overplot=True)
            ss.plottheory(pp_bug)
            ss.plottheory(pp_bug, comp="freq")
            fig1.savefig(pp_bug, format="pdf")
            pp_bug.close()
            pp.close()
            return 0

        dir(ss)
        ss.plotmodes(pp, yscale="log", debug=True, summary=False)
        ss.plotmodes(pp, yscale="symlog", comp="phase", summary=False)
        ss.plotmodes(pp, yscale="symlog", comp="phase", field="rho", summary=False)
        print(dir(ss))

        # ss.plotmodes(pp,yscale='log',comp='phase',clip=True)

        # ss.plotfreq2(pp,xscale='log',yscale='linear',overplot=False)
        for elem in self.meta["evolved"]:
            ss.plotfreq2(
                pp, xscale="log", yscale="symlog", field=elem, overplot=True, trans=True
            )

        # ss.plotfreq2(pp,xscale='log',yscale='symlog',field='rho',overplot=True)

        if quick == True:
            pp.close()
            s.printmeta(pp)

            # plt.savefig(pp, format='pdf')
            return 0

        all_fields = list(set(s.field).union())

        s.plotgamma(pp, xscale="log", yscale="linear", overplot=True, trans=True)

        s.plotgamma(pp, yscale="symlog", xscale="log", overplot=True)
        s.plotgamma(pp, yscale="symlog", xscale="log", field="rho", overplot=True)

        try:
            s.plotfreq2(pp, xscale="log", yscale="linear", overplot=True)
            # s.plotfreq2(pp,xscale='log',yscale='symlog',overplot=False)
            s.plotfreq2(pp, xscale="log", yscale="symlog", field="rho", overplot=True)

            # s.plotfreq2(pp,xscale='log',yscale='linear')
        except:
            print("something terrible")

        s.plotradeigen(pp, yscale="linear")
        # s.plotradeigen(pp,field ='Vi',yscale='linear')
        s.plotradeigen(pp, field="rho", yscale="log")

        pp.close()
        s.printmeta(pp, filename=pdfname)  # append a metadata header



#!/usr/bin/env python
"""
Manually creation of folder structures
To be used with python 2

Dependencies:
-----------
pygraphviz

mmag@fysik.dtu.dk
"""

__authors__ = 'Michael Loeiten'
__email__   = 'mmag@fysik.dtu.dk'
__version__ = '1.0'
__date__    = '21.01.2016'

import pygraphviz as pgv

# Initialize
tree = pgv.AGraph()

# Appendable lists
files     = []
dead_ends = []

# Default node attributes
tree.node_attr['shape']='box'
tree.node_attr['style']='bold'

# Adding nodes and edges
l0 = 'project'
l1 = ['data', 'source\nfiles', 'driver.py']
# Append the files
files.append(l1[1])
files.append(l1[2])
# Add the boxes to the mother node
for box in l1:
    tree.add_edge(l0,box)

l2 = ['solver1', 'solver2',\
      'BOUT.inp', 'run_log.txt']
# Append the files
files.append(l2[2])
files.append(l2[3])
# Add the boxes to the mother node
for box in l2:
    tree.add_edge('data', box)
tree.add_edge('solver2', 'solver2/...')
# Append the dead_end
de = l2[1] + '/...'
dead_ends.append(de)

l3 = ['method1', 'method2', 'solver1/...']
for box in l3:
    tree.add_edge('solver1', box)
tree.add_edge('method2', 'method2/...')
# Append the dead_end
de = l3[2]
dead_ends.append(de)
de = l3[1] + '/...'
dead_ends.append(de)

l4 = ['nout\ntimestep1', 'nout\ntimestep2', 'method1/...']
for box in l4:
    tree.add_edge('method1', box)
tree.add_edge('nout\ntimestep2', 'nout\ntimestep2/...')
# Append the dead_end
de = l4[2]
dead_ends.append(de)
de = l4[1] + '/...'
dead_ends.append(de)

l5 = ['mesh1', 'mesh2', 'nout\ntimestep1/...']
for box in l5:
    tree.add_edge('nout\ntimestep1', box)
tree.add_edge('mesh2', 'mesh2/...')
# Append the dead_end
de = l5[2]
dead_ends.append(de)
de = l5[1] + '/...'
dead_ends.append(de)

l6 = ['additional1', 'additional2', 'mesh1/...']
for box in l6:
    tree.add_edge('mesh1', box)
tree.add_edge('additional2', 'additional2/...')
# Append the dead_end
de = l6[2]
dead_ends.append(de)
de = l6[1] + '/...'
dead_ends.append(de)

l7 = ['grid_file1', 'grid_file2', 'additional1/...']
for box in l7:
    tree.add_edge('additional1', box)
tree.add_edge('grid_file2', 'grid_file2/...')
# Append the dead_end
de = l7[2]
dead_ends.append(de)
de = l7[1] + '/...'
dead_ends.append(de)

l8 = ['BOUT.inp\n(copy)', 'BOUT.log', 'BOUT.dmp',\
      'BOUT.restart', '(source_files\n(copy))', '(grid_file\n(copy))']
# Add l8 to the files list
for cur_file in l8:
    files.append(cur_file)
# Append them to the mother node
for box in l8:
    tree.add_edge('grid_file1', box)


# Change colors for the files
for the_file in files:
    member=tree.get_node(the_file)
#    member.attr['fontcolor'] = 'limegreen'
    member.attr['color'] = 'limegreen'

# Change colors for the dead_ends
for dead_end in dead_ends:
    member=tree.get_node(dead_end)
#    member.attr['fontcolor'] = 'darksalmon'
    member.attr['color'] = 'darksalmon'


# Print the graph
print(tree.string())

# Set layout
tree.layout('dot')
# Write to file
tree.draw('folder_tree.svg')



from __future__ import division
from builtins import zip
from builtins import range
from past.utils import old_div
import numpy as np
from boututils.file_import import file_import
from boututils.surface_average import surface_average
from boutdata.collect import collect
from pylab import plot, show, xlabel, ylabel, figure, legend, gca


path='./data'

gfile='../cbm18_dens8.grid_nx68ny64.nc'

g = file_import(gfile)

var=collect("P", path=path)

sol=surface_average(var, g)
#sol=np.mean(var,axis=3)

p0av=collect("P0", path=path)

q=np.zeros(sol.shape)

for i in range(sol.shape[1]):
    q[:,i]=sol[:,i]+p0av[:,0]


psixy=g.get('psixy')
psi0=g.get('psi_axis')
psix=g.get('psi_bndry')

xarr = psixy[:,0]
xarr = old_div((xarr - psi0), (-psi0 + psix)) #for this grid


fig=figure()

nt=q.shape[1]

plot(xarr, p0av,'k',label='t=0')
plot(xarr,q[:,nt/4],'r',label='t='+np.str(nt/4))
plot(xarr,q[:,nt/2],'b',label='t='+np.str(nt/2))
plot(xarr,q[:,3*nt/4],'g',label='t='+np.str(3*nt/4))
plot(xarr, q[:,-1],'k',label='t='+np.str(nt))

from collections import OrderedDict
handles, labels = gca().get_legend_handles_labels()
by_label = OrderedDict(list(zip(labels, handles)))
legend(list(by_label.values()), list(by_label.keys()))


xlabel(r"$\psi$",fontsize=25)
ylabel(r"$2 \mu_0 <P^2> / B^2$",fontsize=25)

fig.set_tight_layout(True)

show()



from builtins import range
from read_grid import read_grid
from ordereddict import OrderedDict
import numpy as np
import string
import re


def findlowpass(cxxstring):
    ##p1="lowPass\(.*\)"
    p1 = "lowPass\(.*\,(.*)\)"
    maxN = np.array(re.findall(p1, cxxstring))
    # print substrings

    # p2=("[0-9]")

    # maxN = np.array([re.findall(p2,elem) for elem in substrings]).flatten()

    if maxN.size == 0:
        return 100
    else:
        output = int(min(maxN))
        if output == 0:
            return 20
        else:
            return output


def no_comment_cxx(path=".", boutcxx="physics_code.cxx.ref"):
    # print 'no_comment'
    boutcxx = path + "/" + boutcxx
    # boutcxx = open(boutcxx,'r').readlines()
    f = open(boutcxx, "r")
    boutcxx = f.read()
    f.close()

    start = string.find(boutcxx, "/*")
    end = string.find(boutcxx, "*/") + 2

    s = boutcxx[0:start]
    for i in range(string.count(boutcxx, "/*")):
        start = string.find(boutcxx, "/*", end)
        s = s + boutcxx[end + 1 : start - 1]

        end = string.find(boutcxx, "*/", end) + 2

        s = s + boutcxx[end + 1 :]

    # pattern = "\n \s* \(//)* .* \n" #pattern for a section start [All],[Ni], etc
    pattern = "\n+.*;"  # everythin
    pattern = re.compile(pattern)
    result = re.findall(pattern, s)

    # print result

    nocomment = []
    for elem in result:
        # print elem
        elem = elem.lstrip()
        stop = elem.find("//")
        # print start,stop

        if stop > 0:
            nocomment.append(elem[0:stop])
        elif stop == -1:
            nocomment.append(elem)

    # result = pattern.match(val)
    # start = string.find(z,'\n  //')
    # end =string.find(boutcxx,'*/')+2
    # print nocomment

    return nocomment


def get_evolved_cxx(cxxfile=None):
    if cxxfile is None:
        cxxfile = no_comment_cxx()

    # s = cxxfile
    # section_0 = string.find(s,'int physics_run(BoutReal t)')
    # section_1 = string.find(s,'return',section_0)
    # s =  s[section_0:section_1]
    temp = []

    for x in cxxfile:
        i = x.find("bout_solve(")
        # print i,x
        if i != -1:
            comma_i = x[i::].find('"')
            comma_j = x[i::].rfind('"')
            # print x[i+comma_i:i+comma_j+1]
            temp.append(x[i + comma_i + 1 : i + comma_j])

    evolved = []
    [evolved.append(x) for x in set(temp)]
    return np.array(evolved)


def read_cxx(path=".", boutcxx="physics_code.cxx.ref", evolved=""):
    # print path, boutcxx
    boutcxx = path + "/" + boutcxx
    # boutcxx = open(boutcxx,'r').readlines()
    f = open(boutcxx, "r")
    boutcxx = f.read()
    f.close()

    # start by stripping out all comments
    # look at the 1st character of all list elements
    # for now use a gross loop, vectorize later

    start = string.find(boutcxx, "/*")
    end = string.find(boutcxx, "*/") + 2

    s = boutcxx[0:start]
    for i in range(string.count(boutcxx, "/*")):
        start = string.find(boutcxx, "/*", end)
        s = s + boutcxx[end + 1 : start - 1]

        end = string.find(boutcxx, "*/", end) + 2

        s = s + boutcxx[end + 1 :]

    section_0 = string.find(s, "int physics_run(BoutReal t)")
    section_1 = string.find(s, "return", section_0)
    s = s[section_0:section_1]

    tmp = open("./read_cxx.tmp", "w")
    tmp.write(s)
    tmp.close()
    tmp = open("./read_cxx.tmp", "r")

    cxxlist = ""

    for line in tmp:
        if line[0] != "//" and line.isspace() == False:
            cxxlist = cxxlist + line.split("//")[0]

    return cxxlist



from __future__ import print_function
from __future__ import division
from builtins import zip
from builtins import range
from past.utils import old_div

# basic_info return some statistical averages and harmonic info
import numpy as np
import math


def basic_info(data, meta, rescale=True, rotate=False, user_peak=0, nonlinear=None):
    print("in basic_info")
    # from . import read_grid,parse_inp,read_inp,show

    dims = data.shape
    ndims = len(dims)

    mxg = meta["MXG"]["v"]

    if ndims == 4:
        nt, nx, ny, nz = data.shape
        print(nt, nx, ny)
    else:
        print("something with dimesions")

    dc = (
        data.mean(1).mean(1).mean(1)
    )  # there MUST be a way to indicate all axis at once
    amp = abs(data).max(1).max(1).max(1)
    dt = meta["dt"]["v"]

    if rescale:
        amp_o = amp - dc
        fourDamp = np.repeat(amp_o, nx * ny * nz)
        fourDamp = fourDamp.reshape(nt, nx, ny, nz)
        dc_n = old_div(dc, amp_o)
        data_n = old_div(data, fourDamp)

        print(data.shape)
        dfdt = np.gradient(data)[0]
        dfdt = abs(dfdt).max(1).max(1).max(1)

        ave = {"amp": amp, "dc": dc, "amp_o": amp_o, "dfdt": dfdt}

    else:
        print("no rescaling")
        ave = {"amp": amp, "dc": dc}

    if nonlinear is not None:  # add nonlinear part if user provides
        nl = abs(nonlinear[:, mxg : -1.0 * mxg, :, :]).max(1).max(1).max(1)
        nl_norm = (old_div(nl, dfdt)) * dt

        ave["nl"] = nl
        ave["nl_norm"] = nl_norm

    if rotate:
        print("rotate stuff")
        # will need to provide some grid geometry to do this one
    else:
        print("or not")

    # let's identify the dominant modes, for now at every [t,x] slice
    # if the data set is too large we can average over x
    peaks_db = fft_info(
        data, user_peak, meta=meta
    )  # Nt X Nx X (# of loc. max) list of dict

    # print peaks[0]['gamma']
    return peaks_db, ave


def fft_info(
    data,
    user_peak,
    dimension=[3, 4],
    rescale=False,
    wavelet=False,
    show=False,
    meta=0,
    edgefix=False,
):
    import numpy as np
    import math

    print("in fft_inf0")

    dims = data.shape
    ndims = len(dims)

    if ndims == 4:
        nt, nx, ny, nz = data.shape
        print(data.shape)
    else:
        print("something with dimesions")

    # data2 = data
    # if edgefix:
    #     data2 = np.zeros((nt,nx,ny+1,nz+1))

    #     for t in range(nt):
    #         for x in range(nx):
    #             temp = np.append(data[t,x,:,:],[data[t,x,0,:]],0)
    #             data2[t,x,:,:] = np.append(temp,
    #                                        np.transpose([temp[:,0]]),1)

    # dt, k labels for the revelant dimensions

    dt = meta["dt"]["v"]
    dz = meta["dz"]
    # IC = meta['IC']
    ky_max = old_div(ny, 2)
    kz_max = old_div(nz, 2)
    amp = abs(data).max(2).max(2)  # nt x nx

    print("dt: ", dt)

    # print data[0,2,:,:]

    IC = amp[0, :].max()  # intial condition, set
    print(IC)

    fft_data = np.fft.fft2(data)[
        :, :, 0:ky_max, 0:kz_max
    ]  # by default the last 2 dimensions

    power = fft_data.conj() * fft_data
    # print power[0].max(), (IC*(ky_max)*(kz_max))**2

    cross_pow = old_div((fft_data * (np.roll(fft_data, 1, axis=0)).conj()), (ny * nz))

    if rescale:
        fft_data_n = np.fft.fft2(data_n)[:, :, 0:ky_max, 0:kz_max]
        pow_n = np.sqrt((fft_data_n.conj() * fft_data_n).real)

    peaks = [[[] for i in range(nx)] for j in range(nt)]  # a list of dictionaries
    peaks_db = []

    peak_hist = [[0 for i in range(kz_max)] for j in range(ky_max)]  # a 2d bin array

    # for now using a lame 2x loop method

    if user_peak != 0:
        for mem in user_peak:
            print(mem)
            peak_hist[int(mem[0])][int(mem[1])] = abs(
                power.mean(0).mean(0)[int(mem[0]), int(mem[1])]
            )

        # floor =  ((IC*(kz_max*ky_max))**2)/10000

    else:
        for t in range(nt):
            for x in range(nx):
                peaks[t][x] = local_maxima(
                    power[t, x, :, :], 0, floor=(IC * (kz_max * ky_max)) ** 2
                )
                for p in peaks[t][
                    x
                ]:  # looping over each returned peakset at some fixed t,x pair
                    peak_hist[p["y_i"]][
                        p["z_i"]
                    ] += 1  # average across t and x, at least exclude pad
        floor = 0

    # this array is usefull for determining what the dominant modes are
    # but we want to retain the option of observing how the amplitude
    # of any give harmonic varies in space

    peak_hist = np.array(peak_hist)

    # let's find the top N overall powerfull harmnonics
    net_peak = local_maxima(peak_hist, user_peak, bug=False)

    print("net_peak: ", net_peak, user_peak != 0)
    # dom_mode = [{'amp':[],'amp_n':[],'phase':[],'freq':[],'gamma':[]} for x in net_peak]
    dom_mode_db = []

    Bp = meta["Bpxy"]["v"][:, old_div(ny, 2)]
    B = meta["Bxy"]["v"][:, old_div(ny, 2)]
    Bt = meta["Btxy"]["v"][:, old_div(ny, 2)]

    rho_s = meta["rho_s"]["v"]

    L_z = old_div(meta["L_z"], rho_s)
    # L_z =
    L_y = meta["lpar"]  # already normalized earlier in read_inp.py
    L_norm = old_div(meta["lbNorm"], rho_s)

    hthe0_n = 1e2 * meta["hthe0"]["v"] / rho_s  # no x dep
    hthe0_n_x = old_div(L_y, (2 * np.pi))  # no x dep

    print("L_z,Ly: ", L_z, L_y)

    # if user provides the harmo    nic info overide the found peaks

    # thi is where all the good stuff is picked up

    # look at each mode annd pull out some usefull linear measures
    for i, p in enumerate(net_peak):
        # print i,p['y_i'],p['z_i'],fft_data.shape,fft_data[:,:,p['y_i'],p['z_i']].shape

        amp = (
            old_div(np.sqrt(power[:, :, p["y_i"], p["z_i"]]), (kz_max * ky_max))
        ).real

        # print (np.angle(fft_data[:,:,p['y_i'],p['z_i']],deg=False)).real

        phase = -np.array(
            np.gradient(
                np.squeeze(np.angle(fft_data[:, :, p["y_i"], p["z_i"]], deg=False))
            )[0].real
        )  # nt x nx

        gamma_instant = np.array(np.gradient(np.log(np.squeeze(amp)))[0])

        # loop over radaii
        phasenew = []
        gammanew = []
        from scipy.interpolate import interp2d, interp1d
        from scipy import interp

        gamma_t = np.transpose(gamma_instant)
        for i, phase_r in enumerate(np.transpose(phase)):
            gamma_r = gamma_t[i]
            jumps = np.where(abs(phase_r) > old_div(np.pi, 32))
            # print jumps
            if len(jumps[0]) != 0:
                all_pts = np.array(list(range(0, nt)))
                good_pts = (np.where(abs(phase_r) < old_div(np.pi, 3)))[0]
                # print good_pts,good_pts
                # f = interp1d(good_pts,phase_r[good_pts],fill_value=.001)
                # print max(all_pts), max(good_pts)
                # phasenew.append(f(all_pts))
                try:
                    phase_r = interp(all_pts, good_pts, phase_r[good_pts])
                    gamma_r = interp(all_pts, good_pts, gamma_r[good_pts])
                except:
                    "no phase smoothing"
            phasenew.append(phase_r)
            gammanew.append(gamma_r)

        phase = old_div(np.transpose(phasenew), dt)

        gamma_i = old_div(np.transpose(gammanew), dt)

        amp_n = (
            old_div(np.sqrt(power[:, :, p["y_i"], p["z_i"]]), (kz_max * ky_max * amp))
        ).real
        # amp_n = dom_mode[i]['amp_n'] #nt x nx

        # let just look over the nx range
        # lnamp = np.log(amp[nt/2:,2:-2])
        try:
            lnamp = np.log(amp[old_div(nt, 2) :, :])
        except:
            print("some log(0) stuff in basic_info")

        t = dt * np.array(list(range(nt)))  # dt matters obviouslyww
        r = np.polyfit(t[old_div(nt, 2) :], lnamp, 1, full=True)

        gamma_est = r[0][0]  # nx
        f0 = np.exp(r[0][1])  # nx
        res = r[1]
        pad = [0, 0]
        # gamma_est = np.concatenate([pad,gamma_est,pad])
        # f0 = np.concatenate([pad,f0,pad])
        # res = np.concatenate([pad,res,pad])

        # sig = res/np.sqrt((x['nt']-2))
        sig = np.sqrt(old_div(res, (nt - 2)))
        # sig0 = sig*np.sqrt(1/(x['nt'])+ ) # who cares
        sig1 = sig * np.sqrt(old_div(1.0, (nt * t.var())))
        nt = np.array(nt)
        print("shapes ", nt.shape, nt, lnamp.shape, res.shape, gamma_est)
        # print r
        res = 1 - old_div(res, (nt * lnamp.var(0)))  # nx
        res[0:2] = 0
        res[-2:] = 0

        gamma = [gamma_est, sig1, f0, res]

        # gamma_est2 = np.gradient(amp)[0]/(amp[:,:]*dt)
        # gamma_w =  np.gradient(gamma_est2)[0]

        # gamma_i = np.abs(gamma_w).argmin(0) #index of the minimum for any given run
        # for j in range(nx):
        #     gamma_w[0:max([gamma_i[j],nt/3]),j] = np.average(gamma_w)*100000.0

        freq = np.array(
            weighted_avg_and_std(phase[-10:, :], weights=np.ones(phase[-10:, :].shape))
        )

        # gamma = weighted_avg_and_std(
        #         gamma_est2[-5:,:],weights=np.ones(gamma_est2[-5:,:].shape))

        k = [
            [p["y_i"], p["z_i"]],
            [2 * math.pi * float(p["y_i"]) / L_y, 2 * math.pi * p["z_i"] / L_z],
        ]
        # L_y is normalized

        # simple k def, works in drift-instability fine
        # k = [[p['y_i'],p['z_i']],
        #      [(B/Bp)**-1*2*math.pi*float(p['y_i'])/(L_y),(B/Bp)*2*math.pi*p['z_i']/L_z]]

        # k_r = [[p['y_i'],p['z_i']],
        #      [(Bp/B)*2*math.pi*float(p['y_i'])/(L_y),
        #       (B/Bp)*2*math.pi*p['z_i']/L_z]]

        k_r = [
            [p["y_i"], p["z_i"]],
            [
                2 * math.pi * float(p["y_i"]) / (L_y),
                (old_div(B, Bp)) * 2 * math.pi * p["z_i"] / L_z
                + (old_div(Bt, B)) * 2 * math.pi * float(p["y_i"]) / (L_y),
            ],
        ]

        # revised
        # k_r = [[p['y_i'],p['z_i']],
        #      [2*math.pi*float(p['y_i'])/(L_y),
        #       (Bp/B)*2*math.pi*p['z_i']/L_z
        #       - (Bt/Bp)*2*math.pi*float(p['y_i'])/(L_y)]]
        # revised

        # what I think is the most general one, works in drift-instability again
        # seems to work for Bz only helimak, now trying Bp = Bt
        # k = [[p['y_i'],p['z_i']],
        #      [((Bp/B)*float(p['y_i'])/(hthe0_n)) +
        #       2*np.pi*p['z_i']*np.sqrt(1-(Bp/B)**2)/L_z,
        #       2*math.pi*p['z_i']/L_norm -
        #       float(p['y_i'])*np.sqrt(1-(Bp/B)**2)/(hthe0_n)]]
        # k = [[p['y_i'],p['z_i']],
        #      [((Bp/B)*float(p['y_i'])/(hthe0_n)),
        #       2*math.pi*p['z_i']/L_norm]]
        # BOTH SEEM TO PRODOCE SAME RESULTS

        # k = [[p['y_i'],p['z_i']],
        #      [(float(p['y_i'])/(hthe0_n_x)),
        #       2*math.pi*float(p['z_i'])/L_norm]]

        dom_mode_db.append(
            {
                "modeid": i,
                "k": k[1],
                "gamma": gamma,
                "freq": freq,
                "amp": amp,
                "amp_n": amp_n,
                "phase": phase,
                "mn": k[0],
                "nt": nt,
                "k_r": k_r[1],
                "gamma_i": gamma_i,
            }
        )

    return dom_mode_db


# return a 2d array fof boolean values, a very simple boolian filter
def local_maxima(array2d, user_peak, index=False, count=4, floor=0, bug=False):
    from operator import itemgetter, attrgetter

    if user_peak == 0:
        where = (
            (array2d >= np.roll(array2d, 1, 0))
            & (array2d >= np.roll(array2d, -1, 0))
            & (array2d >= np.roll(array2d, 0, 1))
            & (array2d >= np.roll(array2d, 0, -1))
            & (array2d >= old_div(array2d.max(), 5.0))
            & (array2d > floor * np.ones(array2d.shape))
            & (array2d >= array2d.mean())
        )
    else:  # some simpler filter if user indicated some modes
        where = array2d > floor

    # ignore the lesser local maxima, throw out anything with a ZERO
    if bug == True:
        print(array2d, array2d[where.nonzero()], where.nonzero()[0])

    peaks = list(zip(where.nonzero()[0], where.nonzero()[1], array2d[where.nonzero()]))

    peaks = sorted(peaks, key=itemgetter(2), reverse=True)

    if len(peaks) > count and user_peak == 0:
        peaks = peaks[0:count]

    keys = ["y_i", "z_i", "amp"]

    peaks = [dict(list(zip(keys, peaks[x]))) for x in range(len(peaks))]

    return peaks
    # return np.array(peak_dic)


def weighted_avg_and_std(values, weights):
    """
    Returns the weighted average and standard deviation.

    values, weights -- Numpy ndarrays with the same shape.
    """

    if len(values.shape) == 2:
        average = np.average(values, 0)  # , weights=weights)
        variance = old_div(
            (
                np.inner(
                    weights.transpose(), ((values - average) ** 2).transpose()
                ).diagonal()
            ),
            weights.sum(0),
        )
    else:
        average = np.average(values, weights=weights)
        variance = old_div(
            np.dot(weights, (values - average) ** 2), weights.sum()
        )  # Fast and numerically precise

    return [average, variance]



#!/usr/bin/env python3
#
# Generate manufactured solution and sources for FCI test
#

from __future__ import division
from __future__ import print_function

from boutdata.mms import *

from sympy import sin, cos, sqrt

from math import pi

f = sin(y - z) + sin(y - 2 * z)

Lx = 0.1
Ly = 10.0
Lz = 1.0

Bt = 1.0
Bp = 0.05
Bpprime = 0.1

Bpx = Bp + (x - 0.5) * Lx * Bpprime  # Note: x in range [0,1]
B = sqrt(Bpx**2 + Bt**2)


def FCI_ddy(f):
    return (Bt * diff(f, y) * 2.0 * pi / Ly + Bpx * diff(f, z) * 2.0 * pi / Lz) / B


############################################
# Equations solved

print("input = " + exprToStr(f))
print("solution = " + exprToStr(FCI_ddy(f)))



#!/usr/bin/env python

import zoidberg
import matplotlib.pyplot as plt

nx = 5
ny = 6
nz = 7

# Create magnetic field
magnetic_field = zoidberg.field.StraightStellarator(radius = 1.0)

# Create a rectangular grid in (x,y,z)
rectangle = zoidberg.grid.rectangular_grid(nx,ny,nz,
                                           Lx = 1.0, Lz = 1.0, Ly = 10.0,
                                           yperiodic = True)

# Here both the field and and grid are centred at (x,z) = (0,0)
# and the rectangular grid here fits entirely within the coils

maps = zoidberg.make_maps(rectangle, magnetic_field)

# Pick a poloidal slice and the next slice
yslice = 0
pol, ycoord = rectangle.getPoloidalGrid(yslice)
pol_next, ycoord_next = rectangle.getPoloidalGrid(yslice+1)

# Plot the grid points at this poloidal slice
plt.plot(pol.R, pol.Z, 'x')

# Get the coordinates which the forward map corresponds to
R_next, Z_next = pol_next.getCoordinate( maps['forward_xt_prime'][:,yslice,:], maps['forward_zt_prime'][:,yslice,:] )

plt.plot(R_next, Z_next, 'o')

plt.show()

    
